/* FILE: dspmon.c               Copyright (c), Symmetric Research, 1998-1999
 *
 * DSP32C monitor/debugger program.  This is the DOS version.  It requires
 * either the /50 or the /tty command line options, unlike the Win9x/WinNT
 * version which automatically runs in a window.
 *
 *
 */

#include <stdio.h>

#include <stdlib.h>             /* for dos_shell() */
#include <process.h>

#include <ctype.h>
#include <string.h>


#include "dsptools.h"






/* Keyboard defines and keystroke input function:
 *
 * This defines the low level interface to the PC keyboard.
 *
 *
 *
 *
 *
 */

#define F1              0x3B00          /* F1                           */
#define F2              0x3C00          /* F2                           */
#define F3              0x3D00          /* F3                           */
#define F4              0x3E00          /* F4                           */
#define F5              0x3F00          /* F5                           */
#define F6              0x4000          /* F6                           */
#define F7              0x4100          /* F7                           */
#define F8              0x4200          /* F8                           */
#define F9              0x4300          /* F9                           */
#define F10             0x4400          /* F10                          */
#define F11             0x3600          /* F11                          */
#define F12             0x3600          /* F12                          */

#define ESC             0x011B          /* esc                          */
#define ENTER           0x1C0D          /* enter                        */

#define BS              0x0E08          /* backspace                    */
#define CTRL_BS         0x0E7F          /* ctrl backspace               */

#define TAB             0x0F09          /* tab                          */

#define HOME            0x4700          /* home                         */
#define END             0x4F00          /* end                          */
#define PGUP            0x4900          /* page up                      */
#define PGDN            0x5100          /* page down                    */

#define UP              0x4800          /* up arrow                     */
#define DOWN            0x5000          /* down arrow                   */
#define LEFT            0x4B00          /* left arrow                   */
#define RIGHT           0x4D00          /* right arrow                  */

#define SHIFT_UP        0x4838          /* shift up arrow               */
#define SHIFT_DOWN      0x5032          /* shift down arrow             */
#define SHIFT_LEFT      0x4B34          /* shift right arrow            */
#define SHIFT_RIGHT     0x4D36          /* shift left arrow             */

#define CTRL_LEFT       0x7300          /* ctrl left arrow              */
#define CTRL_RIGHT      0x7400          /* ctrl right arrow             */

#define CTRL_PGUP       0x8400          /* ctrl page up                 */
#define CTRL_PGDN       0x7600          /* ctrl page down               */


#define FKEY_HELP       F1              /* function key assigments      */
#define FKEY_SYMBOLS    F2
#define FKEY_QUIT       F3
#define FKEY_DOS_SHELL  F4

#define FKEY_STEP       F5
#define FKEY_HALT       F6



int keyin( void) {

        int c;


        /* Wait for keypress, returning the extended ASCII code. */

        _asm {

                /* Call BIOS int 0x16, func 0x00, wait for keypress. */

                mov     ah,0
                int     0x16
                mov     c,ax

                }


        return( c);


}
int keypress( void) {
        

        /* Check the PC keyboard buffer for a keypress:
         *
         * This function looks to see if a keypress is in the PC's
         * keyboard buffer.  If so, the function throws the keypress
         * away and returns a 1, otherwise 0.
         *
         * It is used below to let the user break out of infinite polling
         * loops when needed.
         *
         *
         *
         *
         *
         */

        _asm {

                /* Call BIOS int 0x16, func 0x01, check keyboard status. */

                mov     ah,1
                int     0x16

                jz      no_key



                /* Call BIOS int 0x16, func 0x00, to clear keypress char. */

                mov     ah,0
                int     0x16

                }

                return( 1);   /* return signal ... */


        no_key:

                return( 0);

}










/* DOS command line usage:
 *
 *
 *
 *
 *
 */

char *DOS_usage[] = {

     "                                                                      ",
     "  dspmon {/50|/tty} [/n] [file.dso [file.dss]] [file.out]             ",
     "                                                                      ",
     "       /50            run in VGA 50 line full screen mode             ",
     "       /tty           run in tty mode                                 ",
     "                                                                      ",
     "       /n             cpu n, where n = 0,1,2,3  (DSPMUL boards only)  ",
     "                                                                      ",
     "       file.dso       SR .dso object file to download                 ",
     "       file.dss       SR .dss symbol table file to use                ",
     "       file.out       WE .out fully located coff file to download     ",
     "                                                                      ",
     "               where {} is required, and [] is optional               ",

     NULL,

   };


void DOS_error( char *msg) {

        int i;


        /* Print the error. */

        if ( !msg )

              printf( "USAGE: Command line usage ...\n");

        else
                printf( "ERROR: %s\n", msg);


        /* Give the DOS command line usage message. */

        for ( i = 0 ; DOS_usage[i] ; i++ )

                printf( "%s\n", DOS_usage[i] );


        /* Quit with an error. */

        exit( 1);


}
void DOS_warning( char *msg) {


        /* Give the user a warning and wait for a key to continue on. */

        printf( "WARNING: %s, any key continues ... ", msg);

            keyin();

                printf( "\n\n");
}








/* Symbol table and lookup functions:
 *
 * These functions read in and manage a .dss symbol table for DSPMON.
 *
 *
 *
 *
 */


/* Symbol types and structure.  These *MUST* agree with DSPASM. */

#define CONS            0x0001
#define BYTE            0x0002
#define WORD            0x0004
#define LONG            0x0008
#define DSPF            0x0010
#define IEEE            0x0020
#define FUNC            0x0040
#define GOTO            0x0080

#define DATA            (BYTE|WORD|LONG|DSPF|IEEE)
#define PROGRAM         (FUNC|GOTO)

#define ALL             (CONS|DATA|PROGRAM)

typedef

    struct { unsigned long value; char *name; int type; } SYMBOL;



/* Symbol table. */

#define MAXSYMBOLS      300

  SYMBOL symbol_table[MAXSYMBOLS]; unsigned nsymbols = 0;



/* Symbol functions. */

void sort_symbols( void) {

        int i, j; SYMBOL *s1, *s2, save;


        /* Sort the symbol table into value order. */

        for ( i = 1 ; i < nsymbols ; i++ ) {

                j = i-1;

                s1 = &symbol_table[i];
                s2 = &symbol_table[j];

                save = *s1;

                while ( j-- >= 0 && save.value < s2->value )

                        *s1-- = *s2--;

                *s1 = save;

                }
}
SYMBOL *symbolic_value( char *name, int *type) {

        unsigned i; SYMBOL *s;

            int strequ( char *src, char *dest);



        /* Do a linear search on the symbol table for the specified name. */

        for ( i = 0 , s = symbol_table ; i < nsymbols ; i++ , s++ ) {

                if ( strequ( name, s->name) )

                        { *type = s->type; return( s); }
                }

        return( NULL);

}
char *symbolic_name( unsigned long value, int type) {

        unsigned imin, imax, itest;

                unsigned long min, max, test;


        /* Binary search for a symbol of the specified value and type. */

        imin = 0;
        imax = nsymbols - 1;

          min = symbol_table[imin].value;
          max = symbol_table[imax].value;


        if ( nsymbols > 0 && min <= value && value <= max ) {

                while ( imin <= imax ) {

                        itest = (imax + imin)/2;

                          test = symbol_table[itest].value;


                        if ( value > test )

                                imin = itest+1;

                        else if ( value < test )

                                imax = itest-1;

                        else {

                             if ( symbol_table[itest].type & type )

                                      return( symbol_table[itest].name);

                                else
                                        return( "");

                                }
                        }
                }

        /* Failed return. */

        return( "");

}
int read_symbols( char *filename) {

        FILE *input;

          SYMBOL *s;

              char buffer[80], name[129]; unsigned long value;

                      int n, type;



        /* Open and read a DSPASM .dss ASCII symbol table file. */

        if ( !(input = fopen( filename, "r")) )

                return( 0);



        /* Check for the proper input file type. */

        if ( fscanf( input, "Global symbol values from %s", buffer) != 1 )

                return( 0);




        /* Read in the symbols. */

               s = symbol_table;
        nsymbols = 0;

  read:

      n = fscanf( input, "%128s %lx %s (%x)\n", name, &value, buffer, &type);

          if ( n == 4 ) {

                s->name  = strdup( name);
                s->value = value;
                s->type  = type;

                  if ( s->name && nsymbols < MAXSYMBOLS )

                          { s++; nsymbols++; goto read; }
                }




        /* Sort the table into value order. */

        sort_symbols();




        /* Check to see if the entire file was read or not. */

        if ( feof( input) )

                return( 1);

        else {

              if ( nsymbols > 0 )

                     return( 1); /* at least some symbols were read */

                else
                        return( 0); /* failed to read any symbols */

                }


}
void clear_symbol_table( void) {


        /* Clear the symbol table. */

        nsymbols = 0;

}







/* DSP32C execution and breakpoint control:
 *
 * The following is for controlling the execution of the DSP32C.  Both
 * the g and s DSPMON commands use this code.  Note that is the DSP32C
 * never reaches a breakpoint, then any PC keybaord hit will interrupt
 * the execution and return control to the monitor.
 *
 *
 * NOTES:
 *
 * * ALL BREAKPOINTING IS DONE IN SOFTWARE
 *
 * The DSP32C does not have hardware breakpointing, so breakpointing must
 * be handled in software.  The scheme is as follows:
 *
 * First, breakpoint code is downloaded to the DSP32C.  Then a goto is
 * planted in the user's code at the breakpoint address.  When execution
 * hits the planted goto and jumps to the breakpoint code, the DSP32C
 * registers are rolled out and the PC is notified that the breakpoint
 * has occurred.
 *
 * After notifying the PC of the break, the DSP32C goes into an infinite
 * loop waiting for a signal from the PC that is should continue on.  To
 * signal the DSP32C to restart, the PC pokes the breakpoint code with a
 * goto instruction returning to the user's code.  Basically, this is
 * self modifying code, and seems to be the only way to have breakpoints
 * without modifying the CAU and DAU flags.
 * 
 * Note that the breakpoint code trashes the value of register r21.  All
 * other CAU and DAU registers and flags are preserved.
 * 
 *
 * * THE BREAKPOINT CODE DSPMON.DSH FROM DSPMON.DSP IS INCLUDED
 *
 * The DSP32C code dspmon.dsp is assembled with the DSPASM /h option to
 * create a hexified code array dspmon.dsh that is included in this file.
 * Including the code as an array in dspmon.exe eliminates the need for
 * having an additional dspmon.dso file around that must be found each
 * time dspmon.exe is started.
 *
 *
 * * THE SYMBOL TABLE DSPMON.DSC FROM DSPMON.DSP IS INCLUDED
 *
 * The DSP32C code dspmon.dsp is also assembled with the DSPASM /sc
 * option to create a symbol table dspmon.dsc for inclusion in this
 * file.  Symbols which need to be known in both dspmon.dsp and dspmon.c
 * are automatically communicated to dspmon.c by this technique.  Note
 * that all dspmon.dsp symbols are automatically given a DSP_ prefix by
 * DSPASM in the C include file dspmon.dsc.
 *
 *
 * * DON'T OVERWRITE THE BREAKPOINT CODE ON THE DSP32C
 *
 * Currently, the DSP32C breakpoint code is downloaded to the internal
 * ram block RAM1 in the DSP32C high memory.  This memory is always present
 * at the same addresses for all DSP32C memory mode configurations,
 * and is probably the best choice for most users.  The base address to
 * download the breakpoint code to is given by the defined constant
 * BREAK_BASE in dspmon.dsp.
 *
 * When the breakpoint code is downloaded, it will overwrite any other
 * program or data there.  User applications should not have program or
 * data in ram block RAM1, and likewise should be careful not to
 * overwrite those areas if breakpointing is to work correctly.
 *
 *
 *
 */


/* Include the hexified code and symbol table from dspmon.dsp. */

#include "dspmon.dsh"
#include "dspmon.dsc"



/* DSP32C register stack. */

struct {

        /* This *MUST* agree with dspmon.dsp. */

        unsigned long cau[0x16];

          union { unsigned long l; float f; } dau[0x4];

             struct cau_flags { unsigned long n, z, v, c; } cau_flags;
             struct dau_flags { unsigned long n, z, v, u; } dau_flags;

        }

           regs;



/* DSP32C instructions. */

#define H_MASK          0x00FF0000L
#define L_MASK          0x0000FFFFL

#define R1_equ( n)      0xC0010000L | ((n&H_MASK)<<5) | (n&L_MASK)
#define Goto( a)        0xA0000000L | ((a&H_MASK)<<5) | (a&L_MASK)
#define Nop             0x00000000L

unsigned long

    users_inst[3],

        break_inst[3] = {  /* instructions to plant at the breakpoint */
        
                Nop,
                Goto( DSP_break),
                Nop

                },

          restart[2];





/* Current execution status and breakpoint address. */

#define HALTED          1       /* execution status values */
#define EXECUTING       2
#define BREAKPOINTING   3

  unsigned execution_status = HALTED; unsigned long breakpoint = 0;




void set_breakpoint( void) {

        unsigned long signal;


        /* Save the user's instructions at the breakpoint. */
        
        DSP32C_Get( breakpoint, users_inst, sizeof( users_inst));


        /* Plant a breakpoint branch in the user's code. */

        DSP32C_Put( break_inst, breakpoint, sizeof( break_inst));


        /* Clear the DSP32C's break signal. */

        signal = 0;

            DSP32C_Put( &signal, DSP_signal, sizeof( signal));


}
void restart_from_breakpoint( void) {


        /* Restart from the DSP32C's breakpoint code. */

        DSP32C_PutLong( Goto( DSP_break_restart), DSP_break_wait);


}
int wait_for_break( void) {

        unsigned long signal; int status;

            void clock( unsigned nterminal);



        /* Wait for the DSP32C's breakpoint signal or a kb interrupt. */

        while ( 1 ) {

          clock( 0x800); /* show the DSPMON busy clock */

            DSP32C_Get( DSP_signal, &signal, sizeof( signal));



                /* If the break occured, get the register dump. */

                if ( signal ) {
                        
                        DSP32C_Get( DSP_regs, &regs, sizeof( regs));

                          status = 1; break;

                        }



                /* Otherwise, check for a kb interrupt. */

                else if ( keypress() )
                        
                            { DSP32C_Halt(); status = 0; break; }

            }


        /* Clear the DSPMON busy clock. */

        clock( 0);


        /* Restore the user's instructions at the breakpoint. */

        DSP32C_Put( users_inst, breakpoint, sizeof( users_inst));


        /* Poke the DSP32C with restart code for the next breakpoint. */

        restart[0] = R1_equ( regs.cau[0]);
        restart[1] =   Goto( breakpoint);

          DSP32C_Put( restart, DSP_break_restart, sizeof( restart));


        /* Return whether the breakpoint occured or got a kbhit. */

        return( status);


}
int execute_breakpoint( void) {

        unsigned status;
        

        /* If this is the first breakpoint, download and start. */
        
        if ( execution_status == HALTED ) {
                
                DSP32C_SrHexLoad( dspmon_dsh);
                
                    set_breakpoint(); DSP32C_Go();
                }

        
        /* Otherwise restart from a previous breakpoint. */
        
        else /* execution_status == BREAKPOINTING */
                
                { set_breakpoint(); restart_from_breakpoint(); }


        /* Wait for the breakpoint. */
        
        return( wait_for_break());

}








/* Low level screen defines, colors, positions, data, and functions:
 *
 * All low level interaction with the screen is done thorough these
 * functions.
 *
 *
 *
 *
 *
 */

#define TTY             1               /* possible video modes */
#define VGA_50          2

    int video_mode;


#define FRAME_BUFFER    0xB8000000L     /* VGA 50 screen defines */

#define SCREEN_HEIGHT   50
#define SCREEN_WIDTH    80

#define BLACK           0x00
#define BLUE            0x01
#define GREEN           0x02
#define CYAN            0x03
#define RED             0x04
#define MAGENTA         0x05
#define BROWN           0x06
#define GREY            0x07

#define INTENSE         0x08

#define COLOR( FORE, BACK)      (((BACK)<<4)|FORE)


void set_video( void) {

        unsigned char color;



        /* Initialize to standard 80x25 text mode. */

        _asm {

                /* Call BIOS int 0x10, func 0x00, set video mode. */

                mov     ah,0x00
                mov     al,0x03

                int     0x10

                }



        /* Program for 50 lines. */

        _asm {

                /* Call BIOS int 0x10, func 0x11[ 0x12], set 8x8 font. */

                mov     ah,0x11
                mov     al,0x12
                mov     bl,0

                int     0x10

                }



        /* Move the hardware cursor off displayable area. */

        _asm {

                /* Call BIOS int 0x10, func 0x02, set cursor position. */

                mov     ah,0x02
                mov     bh,0
                mov     dh,SCREEN_HEIGHT+1
                mov     dl,0

                int     0x10

                }



        /* Toggle the blink/intensity bit to intensity. */

        _asm {

                /* Call BIOS int 0x10, func 0x10[ 0x03], toggle blink bit */

                mov     ah,0x10
                mov     al,0x03
                mov     bl,0            /* enable intensity ... */

                int     0x10

                }



        /* Clear the screen off. */

        color = COLOR( GREY, BLACK);

        _asm {

                /* Call BIOS int 0x10, func 0x06, clear screen. */

                mov     ah,0x06
                mov     al,0x00

                mov     bh,color

                mov     ch,0
                mov     cl,0
                mov     dh,SCREEN_HEIGHT-1
                mov     dl,SCREEN_WIDTH-1

                int     0x10

                }



}
void restore_DOS_screen( void) {



        /* Restore the video mode to standard DOS 80x25, mode 3. */

        _asm {

                /* Call BIOS int 0x10, func 0x00, set video mode. */

                mov     ah,0x00
                mov     al,0x03

                int     0x10

                }



}
int far *offset( int i, int j) {


        /* Compute the offset into the screen for position (i,j). */

        return( (int far *)FRAME_BUFFER + i*SCREEN_WIDTH + j);


}
void gputc( int i, int j, int c, int color) {

        int far *p;


        /* Print the character c at (i,j). */

        p = offset( i, j);

          color <<= 8;

             *p = color | (c&0xFF);


}
void gputs( int i, int j, unsigned char *s, int n, int color) {

        int far *p;


        /* Print the string s at (i,j). */

        p = offset( i, j);

          color <<= 8;

             while ( *s && n )

                  { *p++ = color | *s++; n--; }

                while ( n-- ) *p++ = color | ' ';

}







/* Window allocation and functions:
 *
 * These defines and structures allocate the static character windows
 * for the full screen video mode.
 *
 *
 *
 *
 */


/* Window structure. */

typedef struct window {

        char *title;

        int
                ititle,
                jtitle,
                ctitle_on,
                ctitle_off,

                iscroll,
                jscroll,
                hscroll,
                wscroll,
                cscroll;

        void
                  (*up)( void),
                (*down)( void),
                (*pgup)( void),
                (*pgdn)( void),
                (*home)( void),
                 (*end)( void);

        } WINDOW;



/* Forward declarations of window structure functions. */

void
            display_up( void),
          display_down( void),
          display_pgup( void),
          display_pgdn( void),
          display_home( void),
           display_end( void),

         unassemble_up( void),
       unassemble_down( void),
       unassemble_pgup( void),
       unassemble_pgdn( void),
       unassemble_home( void),
        unassemble_end( void);




/* Display memory window defines. */

#define D_I                     0
#define D_J                     0
#define D_N                     5

#define D_ITITLE                D_I
#define D_JTITLE                D_J
#define D_CTITLE_ON             COLOR( INTENSE|GREY, BLACK)
#define D_CTITLE_OFF            COLOR( GREY, BLACK)

#define D_ISCROLL               D_I+1
#define D_JSCROLL               D_J
#define D_HSCROLL               D_N
#define D_WSCROLL               SCREEN_WIDTH
#define D_CSCROLL               COLOR( GREY, INTENSE|BLACK)


/* Unassembly window defines. */

#define U_I                     6
#define U_J                     0
#define U_N                     31

#define U_ITITLE                U_I
#define U_JTITLE                U_J
#define U_CTITLE_ON             COLOR( INTENSE|GREY, BLACK)
#define U_CTITLE_OFF            COLOR( GREY, BLACK)

#define U_ISCROLL               U_I+1
#define U_JSCROLL               U_J
#define U_HSCROLL               U_N
#define U_WSCROLL               SCREEN_WIDTH
#define U_CSCROLL               COLOR( INTENSE|CYAN, CYAN)

#define U_CBREAKPOINT           COLOR( INTENSE|GREY, GREY)


/* Register window defines. */

#define R_I                     38
#define R_J                     0
#define R_N                     10

#define R_ITITLE                R_I
#define R_JTITLE                R_J
#define R_CTITLE_ON             COLOR( INTENSE|GREY, BLACK)
#define R_CTITLE_OFF            COLOR( GREY, BLACK)

#define R_ISCROLL               R_I+1
#define R_JSCROLL               R_J
#define R_HSCROLL               R_N
#define R_WSCROLL               SCREEN_WIDTH
#define R_CSCROLL               COLOR( GREY, INTENSE|BLACK)


/* Help popup window defines. */

#define H_I                     5
#define H_J                     10
#define H_N                     40

#define H_ITITLE                H_I
#define H_JTITLE                H_J
#define H_CTITLE_ON             COLOR( INTENSE|GREY, BLACK)
#define H_CTITLE_OFF            COLOR( GREY, BLACK)

#define H_ISCROLL               H_I+1
#define H_JSCROLL               H_J
#define H_HSCROLL               H_N
#define H_WSCROLL               SCREEN_WIDTH-2*H_J
#define H_CSCROLL               COLOR( INTENSE|GREY, GREY)


/* Error message popup window defines. */

#define E_I                     41
#define E_J                     5
#define E_N                     3

#define E_ITITLE                E_I
#define E_JTITLE                E_J
#define E_CTITLE_ON             COLOR( INTENSE|GREY, GREY)
#define E_CTITLE_OFF            COLOR( GREY, BLACK)

#define E_ISCROLL               E_I+1
#define E_JSCROLL               E_J
#define E_HSCROLL               E_N
#define E_WSCROLL               SCREEN_WIDTH-2*E_J
#define E_CSCROLL               COLOR( INTENSE|RED, RED)



/* General message popup window defines. */

#define G_I                     41
#define G_J                     5
#define G_N                     3

#define G_ITITLE                G_I
#define G_JTITLE                G_J
#define G_CTITLE_ON             COLOR( INTENSE|GREY, GREY)
#define G_CTITLE_OFF            COLOR( GREY, BLACK)

#define G_ISCROLL               G_I+1
#define G_JSCROLL               G_J
#define G_HSCROLL               G_N
#define G_WSCROLL               SCREEN_WIDTH-2*G_J
#define G_CSCROLL               COLOR( INTENSE|MAGENTA, MAGENTA)



/* Symbol table listing popup window defines. */

#define S_I                     4
#define S_J                     8
#define S_N                     40

#define S_ITITLE                S_I
#define S_JTITLE                S_J
#define S_CTITLE_ON             COLOR( INTENSE|GREY, BLACK)
#define S_CTITLE_OFF            COLOR( GREY, BLACK)

#define S_ISCROLL               S_I+1
#define S_JSCROLL               S_J
#define S_HSCROLL               S_N
#define S_WSCROLL               SCREEN_WIDTH-2*S_J
#define S_CSCROLL               COLOR( INTENSE|MAGENTA, MAGENTA)




/* Static window allocation. */

WINDOW

    display_memory = {

        "Display memory",

            D_ITITLE, D_JTITLE, D_CTITLE_ON, D_CTITLE_OFF,

              D_ISCROLL, D_JSCROLL, D_HSCROLL, D_WSCROLL, D_CSCROLL,

            display_up,
            display_down,
            display_pgup,
            display_pgdn,
            display_home,
            display_end,

            },

    unassemble_memory = {

        "Unassemble memory",

            U_ITITLE, U_JTITLE, U_CTITLE_ON, U_CTITLE_OFF,

              U_ISCROLL, U_JSCROLL, U_HSCROLL, U_WSCROLL, U_CSCROLL,

            unassemble_up,
            unassemble_down,
            unassemble_pgup,
            unassemble_pgdn,
            unassemble_home,
            unassemble_end,

            },


    register_display = {

        "Registers",

            R_ITITLE, R_JTITLE, R_CTITLE_ON, R_CTITLE_OFF,

              R_ISCROLL, R_JSCROLL, R_HSCROLL, R_WSCROLL, R_CSCROLL,

            },


    help_window = {

        "Help (any key continues) ...",

            H_ITITLE, H_JTITLE, H_CTITLE_ON, H_CTITLE_OFF,

              H_ISCROLL, H_JSCROLL, H_HSCROLL, H_WSCROLL, H_CSCROLL,

            },


    error_window = {

        "Error (any key continues, F1 = help)",

            E_ITITLE, E_JTITLE, E_CTITLE_ON, E_CTITLE_OFF,

              E_ISCROLL, E_JSCROLL, E_HSCROLL, E_WSCROLL, E_CSCROLL,

            },


    msg_window = {

        "Message (any key continues, F1 = help)",

            G_ITITLE, G_JTITLE, G_CTITLE_ON, G_CTITLE_OFF,

              G_ISCROLL, G_JSCROLL, G_HSCROLL, G_WSCROLL, G_CSCROLL,

            },


    symbol_window = {

        "Symbol table (arrow keys scroll, other keys continue) ...",

            S_ITITLE, S_JTITLE, S_CTITLE_ON, S_CTITLE_OFF,

              S_ISCROLL, S_JSCROLL, S_HSCROLL, S_WSCROLL, S_CSCROLL,

            };




/* Current window. */

WINDOW *w = NULL;



/* Window functions. */

#define ON      1
#define OFF     0

void wtitle( WINDOW *w, int state) {

        int c;


        /* Display a window title in the specified state. */

        c = ( state == ON ? w->ctitle_on : w->ctitle_off );

            gputs( w->ititle, w->jtitle, w->title, w->wscroll, c);


}
void select_window( WINDOW *wnew) {


        /* Turn the title hightlight off on the current window. */

        if ( w )

                wtitle( w, OFF);


        /* Set the current window to the new one, and highlight it. */

        w = wnew;

          wtitle( wnew, ON);

}

int

  save_area[SCREEN_HEIGHT*SCREEN_WIDTH]; /* save and restore area */


void wsave( WINDOW *w) {

        int i1, j1, ni, nj, i2, j2, i, j, far *src, *dest;


        /* Save the screen area underlying the window w. */

        i1 = w->ititle;
        j1 = w->jtitle;
        ni = w->hscroll;
        nj = w->wscroll;

          i2 = i1+ni;
          j2 = j1+nj-1;


        for ( i = i1 , dest = save_area ; i <= i2 ; i++ ) {

                src = offset( i, j1);

                for ( j = j1 ; j <= j2 ; j++ )

                        *dest++ = *src++;
                }

}
void wrestore( WINDOW *w) {

        int i1, j1, ni, nj, i2, j2, i, j, *src, far *dest;


        /* Restore the screen area underlying the window w. */

        i1 = w->ititle;
        j1 = w->jtitle;
        ni = w->hscroll;
        nj = w->wscroll;

          i2 = i1+ni;
          j2 = j1+nj-1;


        for ( i = i1 , src = save_area ; i <= i2 ; i++ ) {

                dest = offset( i, j1);

                for ( j = j1 ; j <= j2 ; j++ )

                        *dest++ = *src++;
                }
}


#define ICLOCK  49
#define JCLOCK  79

void clock( unsigned nterminal) {

        static unsigned first_call = 1;

            static unsigned tick, c;



        /* Display the clock if the first call. */

        if ( first_call ) {

                first_call = 0; tick = 0;

                  c = COLOR( INTENSE|GREEN, BLACK);

                }



        /* If tick equals the terminal count, toggle the clock mark. */

        if ( tick++ == nterminal ) {

                gputc( ICLOCK, JCLOCK, '', c);

                    tick = 0; c++; if ( c == 0x10 ) c = 1;

                }


        /* Reinitialize and clear the clock mark if nterminal = 0. */

        if ( nterminal == 0 ) {

                gputc( ICLOCK, JCLOCK, ' ', COLOR( BLACK, BLACK));

                    first_call = 1;

                }

}
void popup_error( char *msg, int hotkey) {

        WINDOW *w; int i1, j1, ni, nj, i2, j2, color;

          char buffer[SCREEN_WIDTH];



        /* Popup the error message window. */

        w = &error_window;

           i1 = w->iscroll;
           j1 = w->jscroll;

           nj = w->wscroll;
        color = w->cscroll;


        /* Save the screen area underlying the popup. */

        wsave( w);


        /* Format the message. */

        sprintf( buffer, "    %s ...", msg);



        /* Display the error message popup. */

        wtitle( w, ON);

          gputs( i1++, j1, "", nj, color);

            gputs( i1++, j1, buffer, nj, color);

              gputs( i1++, j1, "", nj, color);



        /* Wait for a keypress other than the hot key to continue. */

        while ( keyin() == hotkey )

                ;


        /* Restore the popup area, and clear off the command line. */

        wrestore( w);


}
void popup_msg( char *msg, int hotkey) {

        WINDOW *w; int i1, j1, ni, nj, i2, j2, color;

          char buffer[SCREEN_WIDTH];



        /* Popup the general message window. */

        w = &msg_window;

           i1 = w->iscroll;
           j1 = w->jscroll;

           nj = w->wscroll;
        color = w->cscroll;


        /* Save the screen area underlying the popup. */

        wsave( w);


        /* Format the message. */

        sprintf( buffer, "    %s ...", msg);



        /* Display the error message popup. */

        wtitle( w, ON);

          gputs( i1++, j1, "", nj, color);

            gputs( i1++, j1, buffer, nj, color);

              gputs( i1++, j1, "", nj, color);



        /* Wait for a keypress other than the hot key to continue. */

        while ( keyin() == hotkey )

                ;


        /* Restore the popup area, and clear off the command line. */

        wrestore( w);

}







/* Command line buffer and display functions:
 *
 * The following defines and functions handle the DSPMON command line.
 *
 *
 *
 *
 *
 *
 */

#define IPROMPT         49                              /* locations */
#define JPROMPT         0

#define ICOMMAND        49
#define JCOMMAND        2

#define CPROMPT         COLOR( INTENSE|GREY, BLACK)
#define CCOMMAND        COLOR(         GREY, BLACK)


char

    *prompt = "";


#define CURSOR  ''



#define MAXCOMMAND      50

  char command_buffer[MAXCOMMAND], *cbuf; int command_length = 0;


void command_clear( void) {


        /* Clear the command line buffer. */

        cbuf = command_buffer; command_length = 0;


        /* Clear the command line display. */

        gputs( IPROMPT, JPROMPT, prompt, strlen( prompt), CPROMPT);

          gputs( ICOMMAND, JCOMMAND, "", MAXCOMMAND+1, CCOMMAND);

            gputc( ICOMMAND, JCOMMAND, CURSOR, CCOMMAND);


}
void command_putc( int c) {

        int i, j;


        /* Add c to the command line buffer, catching special keys. */

        i = ICOMMAND;
        j = JCOMMAND + command_length;

        switch ( c ) {

                case BS:

                        if ( command_length > 0 ) {

                            cbuf--; command_length--;

                                gputc( i, j--, ' ', CCOMMAND);
                                gputc( i, j, CURSOR, CCOMMAND);

                            }

                        break;


                default:

                        c &= 0xFF;

                          if ( command_length < MAXCOMMAND && isprint( c) ) {

                              *cbuf++ = c; command_length++;

                                gputc( i, j++, c, CCOMMAND);
                                gputc( i, j, CURSOR, CCOMMAND);

                            }

                        break;
                }

}
void command_gets( char *msg) {

        int lmsg, i, j, c;



        /* Display a message string in the command area. */

        lmsg = strlen( msg);

            gputs( ICOMMAND, JCOMMAND, msg, MAXCOMMAND+1, CCOMMAND);



        /* Get a response, storing it in the command buffer area. */

        cbuf = command_buffer; command_length = 0;

          i = ICOMMAND;
          j = JCOMMAND + lmsg;

             gputc( i, j, CURSOR, CCOMMAND);


  get_key:

          switch ( c = keyin() ) {

                case BS:

                        if ( command_length > 0 ) {

                            cbuf--; command_length--;

                                gputc( i, j--, ' ', CCOMMAND);
                                gputc( i, j, CURSOR, CCOMMAND);

                            }

                        goto get_key;


                case ENTER:

                        break;


                default:

                   c &= 0xFF;

                     if ( isprint( c) ) {

                             if ( command_length < MAXCOMMAND-lmsg ) {

                                      *cbuf++ = c; command_length++;

                                        gputc( i, j++, c, CCOMMAND);
                                        gputc( i, j, CURSOR, CCOMMAND);

                                    }

                              goto get_key;

                            }
                }


        /* Null terminate the command buffer and return. */

        *cbuf = '\0'; cbuf = command_buffer;

}





/* Screen initialization and updating:
 *
 * Initialization for the screen and updating with new data after a command
 * action.
 *
 *
 *
 *
 */

void
         display( void),
      unassemble( void),
       registers( void);



void initialize_screen( void) {


        /* Set the video mode. */

        set_video();


        /* Initialize the screen display. */

        command_clear();

              display(); unassemble(); registers();


        /* Start off in the display window. */

        select_window( &unassemble_memory);


}
void update_screen( void) {


        /* Update the screen display areas with any new data. */

        display(); unassemble(); registers();

}












/* Command actions for DSPMON:
 *
 * These functions carry out the actions specified by the user from the
 * DSPMON command line.
 *
 *
 *
 *
 *
 *
 *
 */


/* Window management actions. */

void tab_window( void) {

        WINDOW *wnew;


        /* Toggle between the display memory and disassemble windows. */

        if ( w == &display_memory )

                wnew = &unassemble_memory;

        else
                wnew = &display_memory;


        select_window( wnew);

}
void grow_window( void) {


        /* Increase the size of the disassembly window by one line. */

        if ( display_memory.hscroll > 1 ) {

                display_memory.hscroll -= 1;

                unassemble_memory.ititle -= 1;
                   unassemble_memory.iscroll -= 1;
                   unassemble_memory.hscroll += 1;

                display(); unassemble();

                }

}
void shrink_window( void) {


        /* Decrease the size of the disassembly window by one line. */

        if ( unassemble_memory.hscroll > 1 ) {

                display_memory.hscroll += 1;

                unassemble_memory.ititle += 1;
                   unassemble_memory.iscroll += 1;
                   unassemble_memory.hscroll -= 1;

                display(); unassemble();

                }

}
void maximize_window( void) {


        /* Maximize the size of the disassembly window. */

        display_memory.hscroll = 1;
          
          unassemble_memory.ititle = 2;
          unassemble_memory.iscroll = 3;
          unassemble_memory.hscroll = D_HSCROLL + U_HSCROLL - 1;

            display(); unassemble();

}
void minimize_window( void) {


        /* Minimize the size of the disassembly window. */

        display_memory.hscroll = D_HSCROLL + U_HSCROLL - 1;
        
          unassemble_memory.ititle = D_HSCROLL + U_HSCROLL;
          unassemble_memory.iscroll = D_HSCROLL + U_HSCROLL + 1;
          unassemble_memory.hscroll = 1;

            display(); unassemble();

}







/* Display memory. */

struct

    display_data { WINDOW *w; int type; unsigned long start; }

        d = { &display_memory, LONG, 0L };


#define PRINTABLE( c)   (isprint( c) ? c : '.')

char *dprintf( int type, unsigned long addr) {

        int b; static char buf[80]; char *name;

          union {

                unsigned char  b[4];
                unsigned int   w[2];
                unsigned long  l;
                float          f;
                float          i;

                }

                  data;



        /* Read and format DSP32C data. */

        DSP32C_Get( addr, &data, sizeof( data));


        name = symbolic_name( addr, DATA);

          b = sprintf( &buf[0], "%12.12s %06lX: ", name, addr&0xFFFFFFL);


        switch ( type ) {

            case BYTE:

                b += sprintf( &buf[b],

                        "%02X %02X %02X %02X      ASCII [%c%c%c%c]",

                                data.b[0], data.b[1], data.b[2], data.b[3],

                                        PRINTABLE( data.b[0]),
                                        PRINTABLE( data.b[1]),
                                        PRINTABLE( data.b[2]),
                                        PRINTABLE( data.b[3])
                                );
                break;


            case WORD:

                b += sprintf( &buf[b], "%04X %04X", data.w[0], data.w[1]);
                break;


            case LONG:

                b += sprintf( &buf[b], "%08lX", data.l);

                break;

            case DSPF:

                b += sprintf( &buf[b], "%s    DSPF", DSP32C_DspfToA( &data.f));

                break;

            case IEEE:

                b += sprintf( &buf[b], "%s    IEEE", DSP32C_IeeeToA( &data.i));

                break;

                }

        return( buf);

}
void display( void) {

        unsigned long addr; int i1, j1, ni, nj, color, i2, i;



        /* Display the window title bar. */

           i1 = d.w->ititle;
           j1 = d.w->jtitle;
           nj = d.w->wscroll;
        color = (w == &display_memory ? D_CTITLE_ON : D_CTITLE_OFF);

          gputs( i1, j1, d.w->title, nj, color);



        /* Display DSP32C memory. */

         addr = d.start;

           i1 = d.w->iscroll;
           j1 = d.w->jscroll;
           ni = d.w->hscroll;
           nj = d.w->wscroll;
        color = d.w->cscroll;

        i2 = i1+ni-1;

          for ( i = i1 ; i <= i2 ; i++ , addr += 4 )

                gputs( i, j1, dprintf( d.type, addr), nj, color);


}
void display_up( void)

        { d.start -= 4; display(); }

void display_down( void)

        { d.start += 4; display(); }

void display_pgup( void)

        { d.start -= w->hscroll*4; display(); }

void display_pgdn( void)

        { d.start += w->hscroll*4; display(); }

void display_home( void)

        { d.start = 0L; display(); }

void display_end( void)

        { d.start = 0L - w->hscroll*4; display(); }





/* Enter memory data. */

struct

    enter_data { int type; unsigned long addr; } e = { LONG, 0L };


void enter( void) {

        unsigned long addr;

           char *type, msg[MAXCOMMAND]; unsigned n;


        union {

                unsigned int   w;
                unsigned long  l;
                float          f;
                float          i;

                } value;


        int
             space( void),

                hexnum( int ndigits, unsigned long *value),

                    eob( void);




        /* Don't allow byte entry, the PIO is 16 bits wide ... */

        if ( e.type == BYTE ) {

            popup_error( "Byte entry not allowed, try word etc", 0);
                
                return;

                }



        /* Let the user enter some new values. */

        switch ( e.type ) {

                case WORD: type = "WORD"; break;
                case LONG: type = "LONG"; break;
                case DSPF: type = "DSPF"; break;
                case IEEE: type = "IEEE"; break;

                }

        addr = e.addr;



  get_value:

        sprintf( msg, "Value (%s) for 0x%06lX ? ", type, addr&0xFFFFFFL);

           command_gets( msg); space();



             /* Poke the DSP32C with the value. */

             switch ( e.type ) {

                case WORD:

                        if ( hexnum( 4, &value.l) && eob() ) {

                            DSP32C_Put( &value.w, addr, sizeof( unsigned));

                                  addr += 2; break;
                                }

                        return;

                case LONG:

                        if ( hexnum( 8, &value.l) && eob() ) {

                            DSP32C_Put( &value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                                }

                        return;

                case DSPF:

                        n = DSP32C_AToDspf( cbuf, &value.f);

                        if ( n ) {

                          cbuf += n; if ( eob() ) {

                            DSP32C_Put( &value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                               }
                           }

                        return;

                case IEEE:

                        n = DSP32C_AToIeee( cbuf, &value.i);

                        if ( n ) {

                          cbuf += n; if ( eob() ) {

                            DSP32C_Put( &value.l, addr, sizeof( unsigned long));

                                  addr += 4; break;
                                }
                            }

                        return;

                }



                /* Update the screen, and get the next value. */

                update_screen();

                    goto get_value;

}



/* Clear memory. */

struct

    clear_data { unsigned long start, nbytes; } c = { 0L, 0x80L };


void clear( void) {

        unsigned long addr;

          long nwords;

            static unsigned long zero = 0L;



        /* Special case nbytes = 0. */

        if ( c.nbytes == 0 )

                return;



        /* Clear off memory, rounding nbytes up to words. */

          addr = c.start;
        nwords = (c.nbytes+1)/2;


        while ( nwords-- > 0 ) {


            clock( 0x800); /* show the busy clock */


                /* Clear off a location. */

                DSP32C_Put( &zero, addr, sizeof( unsigned)); addr += 2;



                /* Let the user break out with a keypress hit if needed. */

                if ( keypress() ) {

                    popup_error( "Keyboard interrupt, clear not finished", 0);

                         break;

                     }

                }



        /* Reset and clear the busy clock. */

        clock( 0);


        /* Update the screen with the results ... */

        update_screen();

}



/* Unassemble (diassemble) memory. */

struct

    unassemble_data { WINDOW *w; unsigned long start; }

        u = { &unassemble_memory, 0L };


char *uprintf( unsigned long addr) {

        unsigned long value;

          char inst[SCREEN_WIDTH], *name; static char buf[SCREEN_WIDTH];


        /* Disassemble a DSP32C memory location. */

        DSP32C_Get( addr, &value, sizeof( unsigned long));

          DSP32C_Decode( value, inst);

            name = symbolic_name( addr, PROGRAM);

              sprintf( buf, "%12.12s %06lX: %08lX  %-48.48s",

                                          name,
                                          addr & 0x00FFFFFFL,
                                          value,
                                          inst

                                        );
          return( buf);

}
void unassemble( void) {

        unsigned long addr; int i1, j1, ni, nj, i2, i, color;


        /* Display the window title bar. */

           i1 = u.w->ititle;
           j1 = u.w->jtitle;
           nj = u.w->wscroll;

       color = (w == &unassemble_memory ? U_CTITLE_ON : U_CTITLE_OFF);

          gputs( i1, j1, u.w->title, nj, color);



        /* Display DSP32C memory as unassembled mnemonics. */

         addr = u.start;

           i1 = u.w->iscroll;
           j1 = u.w->jscroll;
           ni = u.w->hscroll;
           nj = u.w->wscroll;

        i2 = i1+ni-1;

          for ( i = i1 ; i <= i2 ; i++ , addr += 4 ) {


                /* Put a highlight bar on the breakpoint address. */

                if ( addr == breakpoint )

                    gputs( i, j1, uprintf( addr), nj, U_CBREAKPOINT);


                else
                      gputs( i, j1, uprintf( addr), nj, U_CSCROLL);

              }

}
void unassemble_up( void)

        { u.start -= 4; unassemble(); }

void unassemble_down( void)

        { u.start += 4; unassemble(); }

void unassemble_pgup( void)

        { u.start -= w->hscroll*4; unassemble(); }

void unassemble_pgdn( void)

        { u.start += w->hscroll*4; unassemble(); }

void unassemble_home( void)

        { u.start = 0L; unassemble(); }

void unassemble_end( void)

        { u.start = 0L - w->hscroll*4; unassemble(); }





/* Load an input file. */

void load( char *filename) {


        /* Try to download a SR .dso object file.
         *
         * Note that sr_load automatically does a halt when downloading,
         * and the breakpoint and execution status are updated to reflect
         * that.
         *
         */

        if ( DSP32C_SrLoad( filename, NO_REPORT) ) {

                breakpoint = 0; execution_status = HALTED;

                  clear_symbol_table();

                    u.start = 0;

                }


        /* Try to load a SR .dss symbol table file. */

        else if ( read_symbols( filename) )

                /* downloaded a .dss symbol table file */ ;



        /* Try to download a WE AT&T .out coff object file. */

        else if ( DSP32C_WeLoad( filename, NO_REPORT) ) {

                breakpoint = 0; execution_status = HALTED;

                  clear_symbol_table();

                    u.start = 0;
                }


        /* Unrecognized file format. */

        else {

                popup_error( "Unable to read input file", 0);

                  return;

                }


        /* Update the screen. */

        update_screen();

}





/* Register display. */

struct

    register_data { WINDOW *w; } r = { &register_display };


char                               /* flag and register display formats */

    *flag_fmt = "%-46s CAU(%c%c%c%c) DAU(%c%c%c%c) (0x%06lX)",

       *unknown_flag_fmt = "%-46s CAU(****) DAU(****) (0x%06lX)",


 *register_fmt[] = {

   "         r1 0x%06lX    r7 0x%06lX   r13 0x%06lX   r19 0x%06lX         ",
   "         r2 0x%06lX    r8 0x%06lX   r14 0x%06lX   r20 0x%06lX         ",
   "         r3 0x%06lX    r9 0x%06lX   r15 0x%06lX   r21 0x%06lX  (****) ",
   "         r4 0x%06lX   r10 0x%06lX   r16 0x%06lX   r22 0x%06lX  (ivtp) ",
   "         r5 0x%06lX   r11 0x%06lX   r17 0x%06lX                       ",
   "         r6 0x%06lX   r12 0x%06lX   r18 0x%06lX                       ",
   "                                                                      ",
   "      a0 %-13s (0x%08lX)      a2 %-13s (0x%08lX)                      ",
   "      a1 %-13s (0x%08lX)      a3 %-13s (0x%08lX)                      ",

   },

*unknown_register_fmt[] = {

 "         r1 0x******    r7 0x******   r13 0x******   r19 0x******         ",
 "         r2 0x******    r8 0x******   r14 0x******   r20 0x******         ",
 "         r3 0x******    r9 0x******   r15 0x******   r21 0x******  (****) ",
 "         r4 0x******   r10 0x******   r16 0x******   r22 0x******  (ivtp) ",
 "         r5 0x******   r11 0x******   r17 0x******                        ",
 "         r6 0x******   r12 0x******   r18 0x******                        ",
 "                                                                          ",
 "      a0 ************* (0x********)      a2 ************* (0x********)    ",
 "      a1 ************* (0x********)      a3 ************* (0x********)    ",

 };


char *rprintf( int ifmt) {

        static char buf[2*SCREEN_WIDTH]; /* 2* for sprintf buf growth */

            char num0[32], num1[32], num2[32], num3[32];



        /* Format a line of the DSP32C register display. */

        if ( 0 <= ifmt && ifmt <= 3 ) {

                sprintf( buf, register_fmt[ifmt],

                        regs.cau[ifmt+ 0] & 0xFFFFFFL,
                        regs.cau[ifmt+ 6] & 0xFFFFFFL,
                        regs.cau[ifmt+12] & 0xFFFFFFL,
                        regs.cau[ifmt+18] & 0xFFFFFFL

                        );
                }

        else if ( 4 <= ifmt && ifmt <= 5 ) {

                sprintf( buf, register_fmt[ifmt],

                        regs.cau[ifmt+ 0] & 0xFFFFFFL,
                        regs.cau[ifmt+ 6] & 0xFFFFFFL,
                        regs.cau[ifmt+12] & 0xFFFFFFL

                        );
                }

        else if ( ifmt == 7) {

                strcpy( num0, DSP32C_DspfToA( &regs.dau[0].f));
                strcpy( num2, DSP32C_DspfToA( &regs.dau[2].f));

                sprintf( buf, register_fmt[ifmt],

                        num0, regs.dau[0].l,
                        num2, regs.dau[2].l

                        );
                }

        else if ( ifmt == 8 ) {

                strcpy( num1, DSP32C_DspfToA( &regs.dau[1].f));
                strcpy( num3, DSP32C_DspfToA( &regs.dau[3].f));

                  sprintf( buf, register_fmt[ifmt],

                        num1, regs.dau[1].l,
                        num3, regs.dau[3].l

                        );

                }

        else
                sprintf( buf, register_fmt[ifmt]);


        return( buf);

}
void registers( void) {

        int i1, j1, ni, nj, i2, color, ifmt;

            char buffer[SCREEN_WIDTH];



        /* Display the window title bar, flags and breakpoint. */

           i1 = r.w->ititle;
           j1 = r.w->jtitle;
           nj = r.w->wscroll;
        color = R_CTITLE_OFF;

        if ( execution_status == BREAKPOINTING ) {

                    sprintf( buffer, flag_fmt,

                                r.w->title,

                                regs.cau_flags.n ? 'N' : 'n' ,
                                regs.cau_flags.z ? 'Z' : 'z' ,
                                regs.cau_flags.v ? 'V' : 'v' ,
                                regs.cau_flags.c ? 'C' : 'c' ,

                                regs.dau_flags.n ? 'N' : 'n' ,
                                regs.dau_flags.z ? 'Z' : 'z' ,
                                regs.dau_flags.v ? 'V' : 'v' ,
                                regs.dau_flags.u ? 'U' : 'u' ,

                                breakpoint & 0xFFFFFFL

                                );
                }

        else
                sprintf( buffer, unknown_flag_fmt,

                                      r.w->title, breakpoint&0xFFFFFFL );


        gputs( i1, j1, buffer, nj, color);




        /* Display the DSP32C registers. */

           i1 = r.w->iscroll;
           j1 = r.w->jscroll;

           nj = r.w->wscroll;
        color = r.w->cscroll;


        if ( execution_status == BREAKPOINTING ) {

                for ( ifmt = 0 ; ifmt < 9 ; ifmt++ )

                        gputs( i1++, j1, rprintf( ifmt), nj, color);

                }

        else {

                /* Display the unknown register display. */

                for ( ifmt = 0 ; ifmt < 9 ; ifmt++ )

                    gputs( i1++, j1, unknown_register_fmt[ifmt], nj, color);

                }
}






/* Go, single stepping, and halt commands. */

void update_breakpoint( void) {


        /* Scroll the unassemble window to show the breakpoint. */

        if ( breakpoint < u.start )

                u.start = breakpoint;


          else if ( breakpoint >= u.start + 4*(u.w->hscroll/2) )

                u.start = breakpoint - 4*(u.w->hscroll/2);



        /* Update the screen. */

        update_screen();


}
void go_command( void) {


        /* If already executing, tell the user they should stop first. */
        
        if ( execution_status == EXECUTING )

                popup_error( "Already executing, halt first", 0);



        /* Else if halted start continuous execution from reset. */
        
        else if ( execution_status == HALTED ) {
                
                DSP32C_Go();

                  popup_msg( "Executing from reset until halted", 0);

                    execution_status = EXECUTING;

                }


        /* Else restart continuous execution from a previous breakpoint. */

        else { /* execution_status == BREAKPOINTING */
                
                restart_from_breakpoint();

                  popup_msg( "Continuing execution until halted", FKEY_STEP);

                    execution_status = EXECUTING;

                }



        /* Update the screen. */
    
        update_screen();


}
void kb_interrupt( void) {


        /* Halt if execution was broken with a kb interrupt. */
        
        DSP32C_Halt();
        
            breakpoint = 0; execution_status = HALTED;


        /* Give the user an error message. */

        popup_error( "Execution interrupted with kbhit, DSP32C halted",

                                                        FKEY_STEP);

        /* Update the screen. */
                
        update_breakpoint();


}
void breakpoint_command( unsigned long newbreak) {


        /* If already executing, tell the user they should stop first. */
        
        if ( execution_status == EXECUTING )

                popup_error( "Already executing, halt first", FKEY_STEP);



        /* Otherwise set the breakpoint and execute. */

        else { /* execution_status == HALTED or BREAKPOINTING */

            breakpoint = newbreak;

                if ( execute_breakpoint() )

                      execution_status = BREAKPOINTING;
                        
                else
                        kb_interrupt();
            }
   

        /* Update the screen. */
    
        update_breakpoint();


}
void step_command( void) {

        unsigned long newbreak;


        /* Move the breakpoint one instruction onward and execute. */

        newbreak = breakpoint + 4;
        
                breakpoint_command( newbreak);


}
void halt_command( void) {


        /* Halt the DSP32C, resetting to 0. */

        DSP32C_Halt();
        
            breakpoint = 0; execution_status = HALTED;


        /* Give the user a message. */

        popup_msg( "DSP32C halted", FKEY_HALT);



        /* Update the screen. */

        update_breakpoint();

}







/* Miscellaneous command actions. */

char *help_msg[] = {

        "                                                           ",
        "   Commands                                            ",
        "                                                           ",
        "   d[type] [addr]        display memory                    ",
        "   e[type] [addr]        enter memory value                ",
        "                                                           ",
        "   c [addr] [nbytes]     clear memory                      ",
        "                                                           ",
        "   u [addr]              unassemble memory                 ",
        "                                                           ",
        "   l filename            load a dso, dss, or out file      ",
        "                                                           ",
        "   ?             <F1>    help                              ",
        "   symbols       <F2>    display symbol table              ",
        "   eni                   enable eni interrupt flag         ",
        "                                                           ",
        "   q             <F3>    quit                              ",
        "   dos           <F4>    spawn a dos shell                 ",
        "                                                           ",
        "   ESC                   clear command buffer              ",
        "                                                           ",
        "   g [addr]              go with optional breakpoint       ",
        "   s             <F5>    single step                       ",
        "   h             <F6>    halt and reset pc to 0            ",
        "                                                           ",
        "                                                           ",
        "    where type = (b)yte, (w)ord, (l)ong, dsp(f), (i)eee    ",
        "      addresses may be symbolic if dss file is loaded      ",
        "                                                           ",
        "                                                           ",
        "   Arrow Keys                                          ",
        "                                                           ",
        "          tab, up, down, home, end, pgup, pgdn             ",
        "      ctrl left, ctrl right, ctrl pgup, ctrl pgdn          ",
        "                                                           ",
        "                                                           ",



        NULL,

      };


void help( void) {


    extern WINDOW help_window; /* see window allocation */

        int i1, j1, ni, nj, i2, j2, i, color;

          char **h;



        /* Popup a help window. */

           i1 = help_window.iscroll;
           j1 = help_window.jscroll;
           ni = help_window.hscroll;
           nj = help_window.wscroll;
        color = help_window.cscroll;

        i2 = i1+ni-1;


        wsave( &help_window);

            wtitle( &help_window, ON);


        for ( i = i1, h = help_msg ; i <= i2 && *h ; i++, h++ )

                gputs( i, j1, *h, nj, color);



        /* Wait for a keypress other than the hot key to continue. */

        while ( keyin() == FKEY_HELP )
                
                ;


        /* Restore and continue on. */

        wrestore( &help_window);


}
void symbols( void) {

        WINDOW *w; int i1, j1, ni, nj, i2, j2, color, i, npage;

          SYMBOL *start, *s; char buffer[SCREEN_WIDTH];



        /* List the symbol table, special caseing no symbols. */

        if ( nsymbols == 0 )

                { popup_error( "No symbol table loaded", 0); return; }



        /* Popup the error message window. */

        w = &symbol_window;

           i1 = w->iscroll;
           j1 = w->jscroll;
           ni = w->hscroll;
           nj = w->wscroll;
        color = w->cscroll;


        /* Save the screen area underlying the popup. */

        wsave( w);



        /* Display one page of the symbol table. */

        wtitle( w, ON);

          npage = ni - 2; start = symbol_table;


  display_page:

          s = start;

          i = i1;

            gputs( i++, j1, "", nj, color);

              while ( i < (i1+npage) ) {

                  if ( s < symbol_table+nsymbols ) {

                          sprintf( buffer, "      0x%08lX:   %-31.31s",

                                                          s->value,
                                                          s->name
                                                );

                              gputs( i++, j1, buffer, nj, color);

                          s++;

                          }

                  else
                          gputs( i++, j1, "", nj, color);

                  }

        gputs( i++, j1, "", nj, color);



        /* Wait for a keypress, paging if requested. */

  get_page:

        switch ( keyin() ) {

                case HOME:

                        start = symbol_table;
                        goto display_page;

                case END:

                        start = symbol_table+nsymbols-npage+1;

                        if ( start < symbol_table )

                                start = symbol_table;

                        goto display_page;

                case UP:

                        start--;

                        if ( start < symbol_table )

                                start = symbol_table;

                        goto display_page;

                case DOWN:

                        start++;

                        if ( start > symbol_table+nsymbols )

                                start--;

                        goto display_page;


                case PGUP:

                        start -= npage;

                        if ( start < symbol_table )

                                start = symbol_table;

                        goto display_page;

                case PGDN:

                        start += npage;

                        if ( start > symbol_table+nsymbols )

                                start -= npage;

                        goto display_page;


                case FKEY_SYMBOLS:

                  /* require something other than the hot key to continue */

                        goto get_page;

                }


        /* Restore the popup area. */

        wrestore( w);


}
void dos_shell( void) {

        unsigned char color;

          void initialize_screen( void);





        /* Clear the screen off. */

        color = COLOR( GREY, BLACK);

        _asm {

                /* Call BIOS int 0x10, func 0x06, clear screen. */

                mov     ah,0x06
                mov     al,0x00

                mov     bh,color

                mov     ch,0
                mov     cl,0

                mov     dh,SCREEN_HEIGHT-1
                mov     dl,SCREEN_WIDTH-1

                int     0x10

                }


        /* Move the hardware cursor to the upper left. */

        _asm {

                /* Call BIOS int 0x10, func 0x02, set cursor position. */

                mov     ah,0x02

                mov     bh,0
                mov     dh,0
                mov     dl,0

                int     0x10

                }



        /* Spawn a DOS shell. */

        putenv( "PROMPT=(DSPMON SHELL) $p$g");

          spawnlp( P_WAIT, "command.com", NULL);






        /* Reinitialize the screen after the return from the DOS shell. */

        initialize_screen();




}
void enable_eni( void) {


        /* Enable the PIR interrupt. */

        DSP32C_SetEni( 1);

}

void quit( void) {


        /* Make a clean return to DOS. */

        restore_DOS_screen();

          exit( 0);

}






/* Command line parsing and processing:
 *
 * These functions convert arguments specified on the DSPMON command line
 * to binary values, and call the appropriate actions listed above.
 *
 *
 *
 *
 *
 *
 */

int space( void) {

        char *save;


        /* Skip over any white space in the command buffer. */

        save = cbuf;

          if ( isspace( *cbuf) )

              { while ( isspace( *cbuf) ) cbuf++; return( 1); }


          else
                { cbuf = save; return( 0); }


}
int type( int *flag) {

        struct type { int c, type_flag; } *t;

            static struct type type_table[] = {

                        { 'b',  BYTE, },
                        { 'w',  WORD, },
                        { 'l',  LONG, },
                        { 'f',  DSPF, },
                        { 'i',  IEEE, },

                        {   0,     0, },

                        };


        /* Check for a legal type specifier. */

        for ( t = type_table ; t->c ; t++ ) {

                if ( *cbuf == t->c )

                        { cbuf++; *flag = t->type_flag; return( 1); }

                }


        return( 0);


}
int hexnum( int ndigits, unsigned long *value) {

        char *save; unsigned long num; int d, n;


        /* Check for an n digit hex number. */

        save = cbuf;

          num = 0L; n = 0;



          /* Skip any number of leading 0's. */

          while ( *cbuf == '0' )

                  cbuf++;


          /* Scan for up to ndigits significant hex digits. */

          while ( isxdigit( *cbuf) && n < ndigits ) {

                d = *cbuf++; num <<= 4;


                  if ( '0' <= d && d <= '9' )

                        num += (d - '0');

                  else if ( 'a' <= d && d <= 'f' )

                        num += (d - 'a' + 0xA);

                  else if ( 'A' <= d && d <= 'F' )

                        num += (d - 'A' + 0xA);

                 n++;

                }


        /* Make sure something was read in. */

        if ( cbuf > save && n <= ndigits )

                { *value = num; return( 1); }

        else

                { cbuf = save; return( 0); }

}
int address( unsigned long *value) {

        char *save;

            unsigned long addr; int d, ndigits;

                char name_buffer[MAXCOMMAND], *name; SYMBOL *s; int type;



        /* Check for a symbolic address, requiring leading space. */

        save = cbuf;

        if ( space() && (isalpha( *cbuf) || *cbuf == '_') ) {

            name = name_buffer;

                while ( isalnum( *cbuf) || *cbuf == '_' )

                        *name++ = *cbuf++;

                *name = '\0';



                /* Try to find the name on the symbol table. */

                if ( (s = symbolic_value( name_buffer, &type)) != NULL )

                        { *value = s->value; return( 1); }


                }



        /* Check for a 6 digit hex address, again requiring leading space. */

        cbuf = save;

        if ( space() && hexnum( 6, &addr) ) {

                /* Make sure 24 bits, and round to a word location. */

                  if ( addr <= 0x00FFFFFFL )

                    { addr &= 0xFFFFFFFEL; *value = addr; return( 1); }

                }



        /* Didn't find absolute or symbolic, restore cbuf and return. */

        cbuf = save;

            return( 0);


}
int nbytes( unsigned long *value) {

        char *save;


        /* Check for an 8 digit hex number, requiring leading space. */

        save = cbuf;

          if ( space() && hexnum( 8, value) )

              return( 1);

          else
                { cbuf = save; return( 0); }


}
char *filename( void) {

        char *save, *p;

            static char buf[MAXCOMMAND];


        /* Require a leading space. */

        save = cbuf;

        if ( !space() )

                { cbuf = save; return( 0); }


        /* Check for a filename. */

        p = buf;

            while ( isprint( *cbuf) ) *p++ = *cbuf++;


          *p = '\0';


        /* Make sure something was read in. */

        if ( p > buf )

                return( buf);

        else
                { cbuf = save; return( NULL); }


}
int eob( void) {


        /* Check for the end of the command buffer. */

        space();

          return( *cbuf == '\0' ? 1 : 0 );

}
int eoc( void) {


        /* Check for the end of the command buffer, with an error message. */

        space();


          if ( *cbuf == '\0' )

                return( 1);

          else
                  { popup_error( "Illegal command", 0); return( 0); }


}





/* Command table and flags. */

#define DOS_SHELL               1
#define SYMBOLS                 2
#define ENABLE_ENI              3

#define DISPLAY_MEMORY          4
#define ENTER_MEMORY            5
#define CLEAR_MEMORY            6

#define UNASSEMBLE              7
#define LOAD                    8
#define GO                      9
#define SINGLE_STEP             10
#define HALT                    11

#define HELP                    12
#define QUIT                    13

struct

  command { char *name; int flag; } command_table[] = {

        "dos",          DOS_SHELL,
        "symbols",      SYMBOLS,
        "eni",          ENABLE_ENI,

        "d",            DISPLAY_MEMORY,
        "e",            ENTER_MEMORY,
        "c",            CLEAR_MEMORY,

        "u",            UNASSEMBLE,
        "l",            LOAD,
        "g",            GO,
        "s",            SINGLE_STEP,

        "halt",         HALT,
        "h",            HALT,

        "?",            HELP,

        "quit",         QUIT,
        "q",            QUIT,

        NULL,           0,

      };


int strscan( char *src, char *dest) {


        /* Scan the src to see if it contains all of the dest string. */

        while ( *dest && *src == *dest )

                { src++; dest++; }


        /* If hit the end of dest, then return success. */

        return( *dest ? 0 : 1 );


}
void command_enter( void) {

        struct command *cmd; char *p;

            unsigned long newbreak;



        /* Process the entered command. */

        *cbuf = '\0'; cbuf = command_buffer;


        /* Compare the command to the command table. */

        space();


           if ( *cbuf == '\0' ) /* special case no command */

              { command_clear(); return; }



            for ( cmd = command_table ; cmd->name ; cmd++ ) {

                if ( strscan( cbuf, cmd->name) )

                        { cbuf += strlen( cmd->name); break; }

                }


        /* Parse the rest of the command, and take the action. */

        switch ( cmd->flag ) {

                case DISPLAY_MEMORY:

                        type( &d.type); address( &d.start);

                        if ( eoc() )

                               display();

                        break;

                case ENTER_MEMORY:

                        type( &e.type); address( &e.addr);

                        if ( eoc() )

                                enter();
                        break;

                case CLEAR_MEMORY:

                        address( &c.start); nbytes( &c.nbytes);

                        if ( eoc() )

                                clear();
                        break;

                case UNASSEMBLE:

                        address( &u.start);

                        if ( eoc() )

                                 unassemble();
                        break;

                case LOAD:

                        p = filename();

                        if ( eoc() )

                              load( p);

                        break;

                case GO:

                        if ( address( &newbreak) && eoc() ) {
                                
                                newbreak &= 0xFFFFFFFCL; /* round to 4 byte */

                                    breakpoint_command( newbreak);

                                }

                        else if ( eoc() )

                                go_command();

                        break;

                case SINGLE_STEP:

                        if ( eoc() )

                            step_command();

                        break;

                case HALT:

                        if ( eoc() )

                                halt_command();
                        break;

                case ENABLE_ENI:

                        if ( eoc() )

                                enable_eni();
                        break;

                case HELP:

                        if ( eoc() )

                                help();
                        break;

                case SYMBOLS:

                        if ( eoc() )

                                symbols();
                        break;

                case DOS_SHELL:

                        if ( eoc() )

                                dos_shell();
                        break;

                case QUIT:

                        if ( eoc() )

                                quit();
                        break;

                default:

                        popup_error( "Unknown or illegal command", 0);

                        break;

                }


        /* Clear the command buffer. */
        
        command_clear();


}
void process_commands( void) {

        unsigned c;


        /* Process keyboard input. */

        getkey: switch ( c = keyin() ) {


                /* Process any single keystroke hot key commands. */

                case TAB:

                        tab_window(); break;

                case UP:

                        w->up(); break;

                case DOWN:

                        w->down(); break;

                case PGUP:

                        w->pgup(); break;

                case PGDN:

                        w->pgdn(); break;

                case HOME:

                        w->home(); break;

                case END:

                        w->end(); break;

                case LEFT:

                        grow_window(); break;

                case RIGHT:

                        shrink_window(); break;


                case CTRL_LEFT:
                case CTRL_PGUP:
                        
                        maximize_window(); break;
                        
                case CTRL_RIGHT:
                case CTRL_PGDN:
                        
                        minimize_window(); break;




                case FKEY_HELP:

                        help(); break;

                case FKEY_SYMBOLS:

                        symbols(); break;




                case FKEY_STEP:

                        step_command(); break;

                case FKEY_HALT:

                        halt_command(); break;



                case FKEY_DOS_SHELL:

                        dos_shell(); break;


                case FKEY_QUIT:

                        quit(); break;


                case ESC:

                        command_clear(); break;




                /* Process the entered command. */

                case ENTER:

                        command_enter(); break;



                /* Default to adding the character to the command buffer. */

                default:

                        command_putc( c); break;

                }

            goto getkey;
}






/* Main:
 *
 * Process the DOS command line, initialize the screen, and start processing
 * DSPMON commands.
 *
 *
 *
 *
 *
 */

#define NOT_SET         -1

int

      ncpu = NOT_SET;


struct

    DOS_option { char *name; int *flag, value; } DOS_options[] = {

                "50",       &video_mode,     VGA_50,
                "tty",      &video_mode,     TTY,

                "0",        &ncpu,           0,
                "1",        &ncpu,           1,
                "2",        &ncpu,           2,
                "3",        &ncpu,           3,

                NULL,       NULL,            0,

                };

int strequ( char *src, char *dest) {


        /* Compare two strings to see if they are exactly equal. */

        while ( *src && *dest && *src == *dest )

                { src++; dest++; }


        /* If hit the end of src and dest, then return success. */

        return( *src || *dest ? 0 : 1 );

}


#define MAXFILES        2

void main( int argc, char **argv) {

        struct DOS_option *o;

            char *filename[MAXFILES]; int ifile, nfiles, dso, dss, out;




        /* Process the DOS command line. */

        video_mode = NOT_SET;
              ncpu = NOT_SET;

             ifile = 0;
            nfiles = 0;


        while ( --argc > 0 ) { ++argv;


                /* Check for a command line option, using the table above. */

                if ( **argv == '/' ) { (*argv)++;

                    for ( o = DOS_options ; o->name ; o++ ) {

                            if ( strequ( *argv, o->name ) ) {

                                    if ( *o->flag == NOT_SET )

                                            *o->flag = o->value;
                                    else

                                          DOS_error( "Option set twice ...");
                                  break;

                                }
                            }

                    if ( !o->name )

                            DOS_error( "Illegal command line option ...");
                    }


                /* If not an option, then assume argv is an input filename. */

                else {

                        filename[ifile++] = *argv; nfiles++;

                           if ( nfiles > MAXFILES )

                              DOS_error( "Too many input files ...");
                        }
                }



        /* Require the video mode to be set. */

        if ( video_mode == NOT_SET )

                DOS_error( NULL);



        /* TTY mode not supported yet. */

        if ( video_mode == TTY )

                DOS_error( "tty mode not supported yet");




        /* Initialize the DSP32C board. */

        if ( 0 <= ncpu && ncpu <= 3 )

                DSP32C_SelectCpu( ncpu);


        if ( !DSP32C_Init() )

                    DOS_warning( "Unable to intialize DSP32C board");




        /* Download or read any input files. */

        dso = 0;
        dss = 0;
        out = 0;

        for ( ifile = 0 ; ifile < nfiles ; ifile++ ) {


                if ( DSP32C_SrLoad( filename[ifile], NO_REPORT) )

                        dso++;

                else if ( read_symbols( filename[ifile]) )

                        dss++;

                else if ( DSP32C_WeLoad( filename[ifile], NO_REPORT) )

                        out++;


                else
                      DOS_error( "Illegal input file ...");


                }


        if ( dso > 1 || out > 1 || (dso > 0 && out > 0) )

                DOS_error( "Too many object files on command line ...");

        else if ( dss > 1 )

                DOS_error( "Too many symbol table files on command line ...");




        /* Initialize the screen. */

        initialize_screen();



        /* Start processing DSPMON keyboard commands. */

        process_commands();

}
