/* FILE: download.c             Copyright (c), Symmetric Research, 1998-2000
 *
 * These functions download SR DSPASM .dso and Lucent COFF .out DSP32C
 * object files.
 * 
 * See SrHexload in dspio.c for a lower level downloader that is just
 * as useful.
 *
 *
 *
 * 
 */

#include <stdio.h>

#ifndef SROS_LINUX
#include <io.h>
#endif

#include <fcntl.h>
#include <malloc.h>

#include "dsptools.h"





/* SR DSPASM .DSO DOWNLOADER:
 *
 *
 *
 *
 */

#define SR_MAGIC        0x52535F5FL             /* SR file id, "__SR" */

#define SR_BUFSIZE      2048                    /* multiples of 4 only */
static char sr_download_buffer[SR_BUFSIZE];

#define SR_END_RECORD   0xFFFFFFFFL             /* terminating record */



static unsigned long getl( int fh) {

        unsigned long value;
        

        /* Get a long value from fh. */

        read( fh, &value, 4); return( value);


}

#ifdef SROS_LINUX
#define O_BINARY O_FSYNC
#endif

FUNCTYPE( INT) DSP32C_SrLoad( char *filename, int report) {

        int input;

            unsigned long start, nbytes;

                unsigned n;



        /* Halt DSP32C execution. */
        
        DSP32C_Halt();



        /* Open the input download file and check its id. */

        input = open( filename, O_RDONLY|O_BINARY);

        if ( !input )

                return( 0);
        
        if ( getl( input) != SR_MAGIC )

                { close( input); return( 0); }
        




        /* Download the file a record at a time. */
        
#define SR_MSG0        "       downloading %s (SR format) ... \n"
#define SR_MSG1        "           at 0x%06lX      nbytes = 0x%lX \n"

        if ( report )

                printf( SR_MSG0, filename);
        

        while ( (start = getl( input)) != SR_END_RECORD ) {
                
                nbytes = getl( input);

                if ( report && nbytes )

                        printf( SR_MSG1, start, nbytes);


                /* Read the record a buffer full at a time. */
                
                while ( nbytes ) {

                        n = ( nbytes < SR_BUFSIZE ? nbytes : SR_BUFSIZE );

                          read( input, sr_download_buffer, n);
                        
                              DSP32C_Put( sr_download_buffer, start, n);

                        
                         start += n;
                        nbytes -= n;

                        }
                }
        



        /* Close the input file and return success. */

        close( input);
        return( 1);

}








/* Lucent DSP32CSL COMMON OBJECT FILE FORMAT (COFF) DOWNLOADER.
 *
 *
 *
 *
 *
 *
 *
 *
 */


/* Lucent download buffer. */

#define WEBUFSIZE         2048                    /* multiples of 4 only  */

static char lucent_download_buffer[WEBUFSIZE];



#define WEMAGIC         0x0162          /* WE file id */

typedef struct file_header {

        unsigned magic;
        unsigned nscns;

        long timdat; 
        long symptr;
        long nsyms;

        unsigned opthdr;
        unsigned flags;

        }

          FILE_HEADER;



typedef struct section_header {

        char sname[8];
                
        long paddr;
        long vaddr;
        long size;
        long scnptr;

        long relptr;
        long lnnoptr;
                
        unsigned nreloc;
        unsigned nlnno;
                
        long flags;

        }

          SECTION_HEADER;



static char *we_report[] = {

        "       error -> %s not completely located    \n",

        "       downloading %s (WE format) ...        \n",
        "           at 0x%06lX      section %-8s      \n",

        };



FUNCTYPE( INT) DSP32C_WeLoad( char *filename, int report) {

        int input, i, n, nbytes; long start;

            FILE_HEADER f; SECTION_HEADER *s;



        /* Halt DSP32C execution. */
        
        DSP32C_Halt();



        /* Open the input file and read the file header. */

        if ( !(input = open( filename, /* O_RDONLY|O_BINARY = */ 0x8000 )) )

                return( 0);


        nbytes = read( input, &f, sizeof( FILE_HEADER));

          if ( nbytes != sizeof( FILE_HEADER) || f.magic != WEMAGIC )

                { close( input); return( 0); }




        /* Check to make sure completely located and excutable. */
                
        if ( !(f.flags & /* F_EXEC = */ 0x0002) ) {
                        
                if ( report )
                                
                        printf( we_report[0], filename);
                        
                close( input); return( 0);
                        
                }



        /* Skip past any optional header. */

        lseek( input, f.opthdr, /* SEEK_CUR = */ 1 );



        /* Allocate and read in the section headers. */
                
        nbytes = f.nscns*sizeof( SECTION_HEADER);

           s = (SECTION_HEADER *)malloc( nbytes);


        if ( !s || (read( input, s, nbytes) != nbytes) )
                
                { close( input); return( 0); }



        /* Download the raw data for each section, optionally reporting */

        if ( report )

                printf( we_report[1], filename);


        for ( i = 0 ; i < f.nscns ; i++ , s++ ) {

                if ( report )

                        printf( we_report[2], s->paddr, s->sname);


                /* Seek to the start of the raw data for the section. */

                lseek( input, s->scnptr, /* SEEK_SET = */ 0 );


                /* Read the raw section data a buffer full at a time. */
                
                start = s->paddr; nbytes = s->size;

                while ( nbytes ) {

                        n = ( nbytes < WEBUFSIZE ? nbytes : WEBUFSIZE );

                        read( input, lucent_download_buffer, n);
                        
                                DSP32C_Put( lucent_download_buffer, start, n);
                        
                         start += n; nbytes -= n;

                        }
                }



        /* Close the input file and return success. */

        close( input); return( 1);

}       
