/* FILE: fft.c                  Copyright (c), Symmetric Research, 1998-2000
 *
 * This program computes a FFT.  To run it, execute fft.exe from the DOS
 * command line and some of the results will be displayed on the screen.
 *
 * This example could be easily modified to add in features like
 * floating point to integer data conversion for screen plotting etc.
 * Also see the DSPMATH library for easy to use DSP32C math functions.
 *
 * The DSPASM portion of the code is assembled to a .dsh include file
 * instead of a .dso.  This executable DSP32C code is included as a C
 * array in this file, avoiding having to find a separate .dso file at
 * runtime.
 *
 * This program only relies on the low level DSPIO DLL support library
 * and does not require the full DSPTOOLS.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "dspio.h"





/* DSP32C PORTION OF THE PROGRAM:
 * 
 * 
 * 
 */

#include "fft.dsc"   /* <<< DSP32C symbol table      */
#include "fft.dsh"   /* <<< DSP32C executable code   */





/* ARRAY TO FFT:
 *
 *
 *
 */

#define NARRAY  512

typedef struct { float r, i; } COMPLEX;

COMPLEX

         Input[ NARRAY], Output[ NARRAY];






/* SETUP AND EXECUTE THE DSP32C FFT:
 *
 *
 *
 */

void FFT( COMPLEX *I, COMPLEX *O, long ndimension) {


        /* Copy the input array over to the DSP32C. */

        DSP32C_Put( I, DSP_Array, ndimension*sizeof( COMPLEX));

            DSP32C_Put( &ndimension, DSP_NArray, sizeof( ndimension));




        /* Execute. */

        DSP32C_SetPir( 0);

            DSP32C_Go();

              while ( !DSP32C_Pir() )

                      /* wait for the DSP32C to finish the calculation ... */ ;

        DSP32C_Halt();



        /* Fetch the result into the output array. */

        DSP32C_Get( DSP_Array, O, ndimension*sizeof( COMPLEX));

}





/* DISPLAY HELPER FUNCTION:
 *
 *
 *
 */

void Display( char *msg, COMPLEX *I, COMPLEX *O) {

        int i;
        

        /* Do a short text display of the input and output. */

#define FMT   "  ( %+5.3f, %+5.3f) -> ( %+5.3f, %+5.3f)\n"

        printf( "%s\n\n", msg);

        for ( i = 0 ; i < 19 ; i++ )

            printf( FMT, I[i].r, I[i].i, O[i].r, O[i].i);


}










/* MAIN:
 *
 * Initialize the DSP32C and compute the FFT.
 *
 *
 *
 */

int main( void) {

        int i;



        /* Initialize and download the DSP32C. */

        if ( !DSP32C_Init() )

                { printf( "ERROR: unable to initialize DSP32C\n"); exit( 1); }
        
        if ( !DSP32C_SrHexLoad( fft_dsh) )

                { printf( "ERROR: unable to download fft_dsh\n"); exit( 1); }





        /* Set the value of the complex input array. */
        
        for ( i = 0 ; i < NARRAY ; i++ )

                { Input[i].r = 0.0; Input[i].i = 0.0; }

        Input[1].r = 1.0;
        Input[1].i = 0.0;


        
        /* Compute the FFT. */
        
        FFT( Input, Output, NARRAY);


        /* Display the results. */

        Display( "The first few FFT values are:", Input, Output );


        exit( 0 );
}
