/* FILE: inner.dsp              Copyright (c), Symmetric Research, 1998
 *
 * This is the DSP32C assembler portion of the inner product example.
 * It computes the inner product of the two vectors x and y defined
 * below.
 *
 * Note that the vectors x and y are declared as DSP32C format floats.
 * The DSP32C does its floating point calculations in its own floating
 * point format, and not PC compatible IEEE floats.
 *
 * The processor does have one clock instructions for converting its
 * floating point format to and from IEEE format.  The inner product
 * result computed here is converted to IEEE format before returning it
 * back to the PC.
 *
 *
 *
 */

constant

        PROGRAM         = 0x0000000,
        DATA            = 0x0000200;




/* DSP32C RESIDENT DATA:
 *
 *
 *
 */

<DATA>

dspf
        ZERO = 0.0,
        x[] = { 1.0, 2.0, 3.0 },
        y[] = { 2.0, 3.0, 4.0 };

ieee
        result;




/* INNER PRODUCT PROGRAM:
 *
 *
 *
 */

<PROGRAM>

constant

        BUSY = 0,  /* pir status flag values */
        DONE = 1;


main {


        /* Set the pir register to BUSY. */

        r1 = BUSY; pir = r1;


        /* Zero off the accumulator a0. */

        r1 = ZERO; a0 = *r1;


        /* Point to the x and y arrays and compute the inner product. */

        r1 = x;
        r2 = y;

        do 3
                { a0 = a0 + *r1++ * *r2++; }


        nop; nop; nop; /* <- let the DAU pipeline flush */



        /* Convert a0 to an ieee format float, and save. */

        r3 = result; *r3 = a0 = ieee( a0);



        /* Set the pir register to DONE. */

        r1 = DONE; pir = r1;


        /* Wait for the PC. */

        wait: goto wait;

}
