/* FILE: mmult.c                Copyright (c), Symmetric Research, 1992-2000
 *
 * This program multiplies a vector into a 3x3 matrix, with the vector
 * and matrix coming from the PC portion of the code.  It is an example
 * of more complicated data strutures being shared between the PC and
 * DSP32C.
 *
 * The commands for making the executable mmult.exe are in the make file
 * mmult.  Running mmult.exe from the DOS command line will print out
 * the results.
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>


#include "dsptools.h"




/* Communication area layout.
 *
 * These defines and data structures have the layout of the data to be
 * passed to and returned from the DSP32C.  These data structures *MUST*
 * agree with those in mmult.dsp.
 *
 * The input data has been allocated in a structure to ensure that the C
 * compiler allocates the data sequentially in the C program.  Simply
 * declaring arrays one after another will not necessarily guarantee
 * sequential packed allocation in a C program, whereas it will in
 * DSPASM.
 *
 *       
 */       

#define INPUT           0x100L          /* DSP32C input base address    */

    struct input { float A[3][3], x[3]; } in;


#define OUTPUT          0x200L          /* DSP32C output base address   */

         float y[3];





/* Main program for calling the DSP32C. */

int main() {

        static char *fmt = " %3.1f       %3.1f  %3.1f  %3.1f    %3.1f\n";



        /* Initialize and download the DSP32C. */

        if ( !DSP32C_Init() || !DSP32C_SrLoad( "mmult.dso", NO_REPORT) ) {
                
                printf( "ERROR: unable to initialize or download DSP32C\n");
                exit( 1);
                
                }

        /* Initialize the vector x and matix A. */
        
        in.A[0][0] = 1.0; in.A[0][1] = 1.0; in.A[0][2] = 1.0; 
        in.A[1][0] = 2.0; in.A[1][1] = 2.0; in.A[1][2] = 1.0;
        in.A[2][0] = 1.0; in.A[2][1] = 1.2; in.A[2][2] = 1.4; 

        in.x[0] = 1.0; in.x[1] = 2.0; in.x[2] = 3.0; 


        /* Download the input data. */
        
        DSP32C_Put( &in, INPUT, sizeof( in));


        /* Execute the DSP32C, computing y = A*x. */
        
        DSP32C_Go(); while( !DSP32C_Pir() ); DSP32C_Halt();
        
        
        /* Read the result back. */
        
        DSP32C_Get( OUTPUT, y, sizeof( y));



        /* Print out the input and output. */

        printf( "Mmult result, y = A * x: \n\n");

        printf( fmt, y[0], in.A[0][0], in.A[0][1], in.A[0][2], in.x[0]);
        printf( fmt, y[1], in.A[1][0], in.A[1][1], in.A[1][2], in.x[1]);
        printf( fmt, y[2], in.A[2][0], in.A[2][1], in.A[2][2], in.x[2]);
        
        
        printf( "\n  y should be: ( 6.0, 9.0, 7.6)\n");

        exit( 0 );
}
