/* FILE: sqrt.dsp               Copyright (c), Symmetric Research, 1992
 *
 * This program computes the function sqrt(x), where x is a positive
 * number.
 *
 * Like the program sine.dsp, the algorithm used here has been taken
 * from the AT&T Applications Software Library Reference manual.  In
 * particular, the algorithm listed on page _sqr_1 has been used.  The
 * source has been modified somewhat to use some of the syntax supported
 * by DSPASM.
 *
 * Also, the computation has been placed inside an infinite goto loop,
 * so that when it is run in DSPMON you can poke memory and immediately
 * see the result without halting or restarting DSP32C.
 *
 * Finally, the program uses forward referencing and should be assembled
 * with the DSPASM /2 option.
 *
 *
 */

sqrt {

        register

                c       = r1,
                x       = r2,
                exp     = r3,
                iexp    = r4,
                temp    = r5,
                y       = r6;


    compute:

        /* Point to the input value, and place to store the result. */

        x = value; a0 = *x; y = result;


        /* Compute the a0 = sqrt( a0). */

        x = sqrt_x;

                *x = a0 = a0;

        c = sqrt_coeff; temp = sqrt_temp;

                *temp++ = a1 = *c++;
                *temp-- = a1 = *c++;

        exp = *x.b;

                iexp = -exp; *temp = iexp.b;

        a3 = *x * *temp;

         exp >>= 1; if ( cs ) { temp += 4; }

          a2 = a3 * a3;
          a1 = a3 * *c++;
          a1 = a1 + *c++;
          a1 = a1 + a2 * *c++;
          a2 = a3 * *c++;
          a2 = a2 + *c++;
          a1 = a2 + a2 * a1;
          a3 = a3 * *c++;
          exp += 0x40;
          a1 = a3 * a1;
          a2 = a1 * a1;
          *temp = exp.b;
          a1 = a1 * *temp;
          a2 = *c - a2 * a3;
          nop;
          nop;

        *y = a0 = a1 * a2;  /* save the result off to memory */


   goto compute;


}

dspf

     value = 2.1, result = 0.0,

        sqrt_coeff[] = {

                         1.000000000,
                         1.414213600,

                        -0.382891660,
                         1.187877720,
                         0.049693281,
                        -1.921554740,
                         2.066790900,
                         0.500000000,
                         1.500000100,

                        },

            sqrt_x, sqrt_temp[2]; /* additional temporary variables */
