/* FILE: diag.c                 Copyright (c), Symmetric Research, 1996-2000
 *
 * This diagnostic program checks for proper installation of a SR DSPA64
 * system, and acquisition on the channel specified on the command line.
 *
 *
 *
 *
 *
 *
 */

#include <stdio.h>

#include "dspio.h"






/* Maximum number of channels on the DSPA64. */

#define MAXCHANNEL      64







/* Diagnostic status messages and helper function:
 *
 *
 *
 *
 *
 */

char *chop( char *s) {

        char *save;

        save = s;

        while ( *s )

                s++;


        if ( *--s == '\n' ) {

                while ( *--s == ' ' )

                        /* back up */ ;

                *++s = '\n';
                *++s = '\0';

                }


        return( save);


}
void print_msg( char *msg[]) {

        int i;


        /* Printf the message, chopping off trailing white space. */

        for ( i = 0 ; msg[i] ; i++ )

                printf( chop( msg[i]));


}


char *LEADER[] = {

    "SR DSPA64/DSPHLF diagnostic software tests rev {01/21/00}          \n",
    "                                                                   \n",

    NULL,

    };


char *INIT_SUCCESS[] = {

    ">>> DSPHLF coprocessor board successfully initialized ...            \n",

    NULL,

    };


char *INIT_FAILED[] = {

    "                                                                     \n",
    ">>> Failed DSPHLF initialization!                                    \n",
    "                                                                     \n",
    "   Possible problems are:                                            \n",
    "                                                                     \n",
    "     1. Maybe no DSPHLF coprocessor board is installed ...           \n",
    "     2. Make sure that DSPHLF base IO address switch is either:      \n",
    "                                                                     \n",
    "       * set to the 0x280 default                                    \n",
    "       * or agrees with the DSP32C_BASEIO DOS environment variable   \n",
    "                                                                     \n",
    "     3. Maybe NT or Linux device driver has not been installed ...   \n",
    "                                                                     \n",
    "   see the Users manuals for more information on switch settings     \n",
    "                                                                     \n",
    "                                                                     \n",

    NULL,

    };


char *DOWNLOAD_SUCCESS[] = {

    ">>> DSPHLF diagnostic software successfully downloaded ...           \n",

    NULL,

    };


char *DOWNLOAD_FAILED[] = {

    ">>> Failed DSPHLF diagnostic software download!                      \n",

    NULL,

    };  


char *TEST_VALUE_SUCCESS[] = {

    ">>> DSPHLF id test value successfully read ...                       \n",

    NULL,

    };


char *TEST_VALUE_FAILED[] = {

    "Failed DSPHLF id test value read ...                                 \n",

    NULL,

    };


char *IBF_SUCCESS[] = {

    ">>> DSPA64 IBF signal successfully received ...                      \n",

    NULL,

    };


char *IBF_FAILED[] = {

    "                                                                    \n",
    ">>> Failed DSPA64 IBF test ...                                      \n",
    "                                                                    \n",
    " Things to check:                                                   \n",
    "                                                                    \n",
    "  * Does the ribbon cable connect the DSPA64 and DSPHLF ?           \n",
    "  * Is the DSPA64 properly supplied with power, +/-12 and +5 ?      \n",
    "                                                                    \n",
    "      THE DSPA64 MUST BE POWERED UP BEFORE IT CAN RETURN DATA ...   \n",
    "                                                                    \n",

    NULL,

    };



char *TOTAL_SUCCESS[] = {

    "                                                                     \n",
    "                                                                     \n",
    "SUCCESS ... all DSPA64/DSPHLF diagnostic tests completed ...         \n",

    NULL,

    };











/* DSP32C includes and helper functions:
 *
 *
 *
 *
 *
 */

#include "diag.dsc"
#include "diag.dsh"


#define DSP_GOTO        0xA0000000L


void poke_DSP_RESET( unsigned long func) {

        unsigned long reset;



        /* Poke the DSP32C reset location with the instruction goto func. */

        reset = (DSP_GOTO | func);

          DSP32C_Put( &reset, DSP_RESET, sizeof( reset));



}














/* Main program:
 *
 *
 *
 *
 *
 */

int main( int argc, char *argv[]) {

        int channel, signal, value;
        
            unsigned long test_value;



                


        /* Process the DOS command line, and check the channel number. */

        if ( argc != 2 || !sscanf( argv[1], "%d", &channel) )

                { printf( "USAGE: diag channel\n"); exit( 1); }


        if ( channel < 0 || MAXCHANNEL <= channel ) {

                printf( "ERROR: channel must be in the range [0..63]\n");

                exit( 1);

                }




        /* Display the leader message. */

        print_msg( LEADER);





        /* Initialize the DSPHLF board. */

        if ( DSP32C_Init() )

                print_msg( INIT_SUCCESS);

        else
                { print_msg( INIT_FAILED); exit( 2); }

        



        /* Download the DSPHLF diagnostic software. */
        
        if ( DSP32C_SrHexLoad( diag_dsh) )
                
                print_msg( DOWNLOAD_SUCCESS);

        else
                { print_msg( DOWNLOAD_FAILED); exit( 2); }






        

        /* Read a test id value from the DSPHLF. */

        DSP32C_Get( DSP_TEST_VALUE, &test_value, sizeof( test_value));


        if ( test_value == 0x11223344 )

                print_msg( TEST_VALUE_SUCCESS);

        else
                { print_msg( TEST_VALUE_FAILED); exit( 2); }

                





        
        /* Test for the DSPA64 IBF data ready signal. */

        poke_DSP_RESET( DSP_test_ibf);

        DSP32C_SetPir( 0);

        DSP32C_Go();

                while ( !(signal = DSP32C_Pir()) )

                        ;

                DSP32C_Halt();


        if ( signal == DSP_SUCCESS )

                print_msg( IBF_SUCCESS);

        else
                { print_msg( IBF_FAILED); exit( 2); }









        /* Acquire data from a single channel:
         *
         * The user is on their own to decide if this test is successful !
         *
         *
         */

        poke_DSP_RESET( DSP_acquire);

        DSP32C_Put( &channel, DSP_CHANNEL, sizeof( channel));


        DSP32C_SetPir( 0);

        DSP32C_Go();

                while ( !DSP32C_Pir() )

                        ;

                DSP32C_Halt();


        DSP32C_Get( DSP_VALUE, &value, sizeof( value));


        printf( ">>> Value from channel[%d] = %+06d \n", channel, value);







        /* Finished testing ... make a clean exit to DOS. */

        print_msg( TOTAL_SUCCESS);

        exit( 0);

}
