/* FILE: dspa64.dsp             Copyright (c), Symmetric Research, 1997
 *
 * This is the DSP32C portion of the DSPA64 acquisition library.
 *
 * See dspa64.c for more details on the PC portion of the program.
 *
 *
 *
 */





/* CONSTANTS:
 *
 * These defines set the program memory locations.
 *
 *
 */

constant

        /* Program locations. */

        RESET           = 0x000000,
        DATA            = 0x001000;







/* RESET/MAIN ENTRY POINT:
 *
 * DSP32C execution begins here after the PC issues a go() command.
 *
 *
 *
 *
 */


<RESET>

main {

        register

                waits           = r1,

                config          = r2,
                channel         = r3,

                value           = r4,

                filter_scale    = r5,
                filter_coeff    = r6,

                count           = r7,
                ncount          = r8,

                buffer1         = r9,
                buffer1_end     = r10,

                buffer2         = r11,
                buffer2_end     = r12,

                buffer1_ready   = r13,
                buffer2_ready   = r14;





        /* Program the DSP32C for 0 wait states. */

        waits = 0; pcw = waits;



        /* Initialize the serial port. */

        ioc = ILEN_16 | IN_MSB | OLEN_08 | OUT_MSB;






        /* Read 16 bit integer values from the DSPA64 channels:
         *
         * First any pointers and values are initialized.  Then the
         * polling loop checks to see if data has come in from the
         * DSPA64 board, processing each sample.
         *
         * Always make sure any code inside the polling loop can
         * complete before the next sample comes in.
         *
         *
         *
         */

               ncount = *NCOUNT;

              buffer1 = *BUFFER1;
          buffer1_end = *BUFFER1_END;

              buffer2 = *BUFFER2;
          buffer2_end = *BUFFER2_END;




               config = C00;
                value = VALUES;

         filter_scale = FILTER_SCALE;
         filter_coeff = FILTER_COEFF;



        buffer1_ready = BUFFER1_READY;    /* ... pir flags ... */
        buffer2_ready = BUFFER2_READY;


                                                /* convert to dspf floats */

        *filter_scale = a0 = dsp( *filter_scale);
        *filter_coeff = a0 = dsp( *filter_coeff);





POLL_IBF:

        if ( ibf ) {  /* then a value came in from the DSPA64 ... */



                /* Set a new mux channel. */

                channel = *config++;
                   obuf = channel.b;





                /* Get and filter the new data value:
                 *
                 * See the program filter.dsp for more discussion about
                 * the filter equations.
                 *
                 *
                 */

                a1 = float( ibuf);

                        nop;
                        nop;
                        nop;

                a1 = *filter_scale * a1;

                        nop;

                  a0 = a1 + *filter_coeff * *config;

                        nop;


                *config++ = a0 = a0;

                        nop;






                /* Check the channel decimation count:
                 *
                 * Save the filtered value if the decimation count for
                 * the channel has been reached.
                 *
                 *
                 *
                 *
                 */

                count = *config;

                  if ( count++ == ncount ) {



                        /* Save the filtered value. */

                        *value++ = a2 = int( a0);




                        /* Check for circular buffer full conditions:
                         *
                         * Send a pir signal to the PC if full and reset
                         * the buffer pointer if necessary.
                         *
                         *
                         *
                         */

                        if ( value == buffer1_end )

                                pir = buffer1_ready;


                        else if ( value == buffer2_end ) {


                                pir = buffer2_ready;


                                /* Reset buffer pointer */

                                value = VALUES;

                                }




                        /* Reset the count. */

                        count = 0;

                        }


                *config++ = count;









                /* Move on to the next channel configuration. */

                config = *config;

                }



        goto POLL_IBF;

}







/* DSPA64 SYSTEM PARAMETERS:
 *
 * The followings system paramters are set up by the PC before execution
 * is started to control the sampling rate, channels scanned, etc.  The
 * layout of this data MUST agree with the DSPA64 system parameters data
 * structure in dspa64.c.
 *
 * Note that the configuration list is at the core of the program.  It
 * has the channel number to use, current filtered value and decimation
 * count, and pointer to the next configuration entry.  The DSP moves on
 * from one configuration entry to another for each sample received from
 * the DSPA64.
 *
 * The default example values listed here are included so dspa64.dsp
 * can be run stand alone by hand in DSPMON if needed.  They have been
 * set to scan channels 0 and 1.  Note that NCHANNEL and NTRACE are not
 * actively used anywhere in dspa64.dsp.
 *
 *
 *
 */

constant

        WORDSIZE        = 2,                    /* channel setup */
        NCHANNEL        = 2,
        NTRACE          = 0x200,

        NBUFFER         = (NCHANNEL*NTRACE)*WORDSIZE,


            NOT_READY   = 0x0,                  /* pir signals */
        BUFFER1_READY   = 0x1,
        BUFFER2_READY   = 0x2;






constant SYSTEM = . ;

  /* from here on must agree with dspa64.c SYSTEM parameters struct ... */


ieee

        FILTER_SCALE    = 0.10000000,     /* <- should be (1-FILTER_COEFF) */
        FILTER_COEFF    = 0.90000000;


long
        NCOUNT          = 0,

        BUFFER1         = VALUES + 0*NBUFFER,
        BUFFER1_END     = VALUES + 1*NBUFFER,

        BUFFER2         = VALUES + 1*NBUFFER,
        BUFFER2_END     = VALUES + 2*NBUFFER,


        C00[] = { 0x00,  0x00000000,  0x00000000,  C01 },
        C01[] = { 0x01,  0x00000000,  0x00000000,  C00 },
        C02[] = { 0x02,  0x00000000,  0x00000000,  C03 },
        C03[] = { 0x03,  0x00000000,  0x00000000,  C04 },
        C04[] = { 0x04,  0x00000000,  0x00000000,  C05 },
        C05[] = { 0x05,  0x00000000,  0x00000000,  C06 },
        C06[] = { 0x06,  0x00000000,  0x00000000,  C07 },
        C07[] = { 0x07,  0x00000000,  0x00000000,  C08 },
        C08[] = { 0x08,  0x00000000,  0x00000000,  C09 },
        C09[] = { 0x09,  0x00000000,  0x00000000,  C0A },
        C0A[] = { 0x0A,  0x00000000,  0x00000000,  C0B },
        C0B[] = { 0x0B,  0x00000000,  0x00000000,  C0C },
        C0C[] = { 0x0C,  0x00000000,  0x00000000,  C0D },
        C0D[] = { 0x0D,  0x00000000,  0x00000000,  C0E },
        C0E[] = { 0x0E,  0x00000000,  0x00000000,  C0F },
        C0F[] = { 0x0F,  0x00000000,  0x00000000,  C10 },
        C10[] = { 0x10,  0x00000000,  0x00000000,  C11 },
        C11[] = { 0x11,  0x00000000,  0x00000000,  C12 },
        C12[] = { 0x12,  0x00000000,  0x00000000,  C13 },
        C13[] = { 0x13,  0x00000000,  0x00000000,  C14 },
        C14[] = { 0x14,  0x00000000,  0x00000000,  C15 },
        C15[] = { 0x15,  0x00000000,  0x00000000,  C16 },
        C16[] = { 0x16,  0x00000000,  0x00000000,  C17 },
        C17[] = { 0x17,  0x00000000,  0x00000000,  C18 },
        C18[] = { 0x18,  0x00000000,  0x00000000,  C19 },
        C19[] = { 0x19,  0x00000000,  0x00000000,  C1A },
        C1A[] = { 0x1A,  0x00000000,  0x00000000,  C1B },
        C1B[] = { 0x1B,  0x00000000,  0x00000000,  C1C },
        C1C[] = { 0x1C,  0x00000000,  0x00000000,  C1D },
        C1D[] = { 0x1D,  0x00000000,  0x00000000,  C1E },
        C1E[] = { 0x1E,  0x00000000,  0x00000000,  C1F },
        C1F[] = { 0x1F,  0x00000000,  0x00000000,  C20 },
        C20[] = { 0x20,  0x00000000,  0x00000000,  C21 },
        C21[] = { 0x21,  0x00000000,  0x00000000,  C22 },
        C22[] = { 0x22,  0x00000000,  0x00000000,  C23 },
        C23[] = { 0x23,  0x00000000,  0x00000000,  C24 },
        C24[] = { 0x24,  0x00000000,  0x00000000,  C25 },
        C25[] = { 0x25,  0x00000000,  0x00000000,  C26 },
        C26[] = { 0x26,  0x00000000,  0x00000000,  C27 },
        C27[] = { 0x27,  0x00000000,  0x00000000,  C28 },
        C28[] = { 0x28,  0x00000000,  0x00000000,  C29 },
        C29[] = { 0x29,  0x00000000,  0x00000000,  C2A },
        C2A[] = { 0x2A,  0x00000000,  0x00000000,  C2B },
        C2B[] = { 0x2B,  0x00000000,  0x00000000,  C2C },
        C2C[] = { 0x2C,  0x00000000,  0x00000000,  C2D },
        C2D[] = { 0x2D,  0x00000000,  0x00000000,  C2E },
        C2E[] = { 0x2E,  0x00000000,  0x00000000,  C2F },
        C2F[] = { 0x2F,  0x00000000,  0x00000000,  C30 },
        C30[] = { 0x30,  0x00000000,  0x00000000,  C31 },
        C31[] = { 0x31,  0x00000000,  0x00000000,  C32 },
        C32[] = { 0x32,  0x00000000,  0x00000000,  C33 },
        C33[] = { 0x33,  0x00000000,  0x00000000,  C34 },
        C34[] = { 0x34,  0x00000000,  0x00000000,  C35 },
        C35[] = { 0x35,  0x00000000,  0x00000000,  C36 },
        C36[] = { 0x36,  0x00000000,  0x00000000,  C37 },
        C37[] = { 0x37,  0x00000000,  0x00000000,  C38 },
        C38[] = { 0x38,  0x00000000,  0x00000000,  C39 },
        C39[] = { 0x39,  0x00000000,  0x00000000,  C3A },
        C3A[] = { 0x3A,  0x00000000,  0x00000000,  C3B },
        C3B[] = { 0x3B,  0x00000000,  0x00000000,  C3C },
        C3C[] = { 0x3C,  0x00000000,  0x00000000,  C3D },
        C3D[] = { 0x3D,  0x00000000,  0x00000000,  C3E },
        C3E[] = { 0x3E,  0x00000000,  0x00000000,  C3F },
        C3F[] = { 0x3F,  0x00000000,  0x00000000,  C00 };








/* STORAGE AREA FOR INCOMING DATA VALUES FROM THE DSPA64:
 *
 * The multiplexed data for all of the scanned channels is stored in
 * this area.
 *
 * This area is maintained as a large circular buffer, where the DSP
 * wraps around from the end to the beginning when full.
 *
 *
 * From the PC's point of view this area is maintained as two buffers,
 * with storage ping ponging between the first and second halves.  The
 * size of each buffer that the PC accesses is NCHANNEL*NTRACE words,
 * half of the full circular buffer.
 *
 *
 *
 */

<DATA>

word
        VALUES[ /* NVALUES = 2*NCHANNEL*NTRACE */ ];











/* SERIAL PORT IOC CONTROL REGISTER BIT ASSIGMENTS:
 *
 * These constants define the bit assigments for the serial port IOC
 * configuration control register.  They provide convenient mnemonic names
 * for the bits.
 *
 *
 */

constant

        ILEN_00         = 0x0 << 0x06,          /* input length         */
        ILEN_08         = 0x1 << 0x06,
        ILEN_16         = 0x2 << 0x06,
        ILEN_32         = 0x3 << 0x06,

        OLEN_00         = 0x0 << 0x0A,          /* output length        */
        OLEN_08         = 0x1 << 0x0A,
        OLEN_16         = 0x2 << 0x0A,
        OLEN_32         = 0x3 << 0x0A,

        IN_LSB          = 0x0 << 0x10,          /* input lst first      */
        IN_MSB          = 0x1 << 0x10,          /* input msb first      */

        OUT_LSB         = 0x0 << 0x11,          /* output lsb first     */
        OUT_MSB         = 0x1 << 0x11;          /* output msb first     */

