/* FILE: dspa64.h                Copyright (c), Symmetric Research, 1998-2000
 *
 * SR DSPA64 low level/DLL interface library include file.
 *
 *
 *
 *
 *
 *
 */




/* FUNCTION AND VARIABLE TYPES:
 *
 * These defines control the function and variable types for the SR DSPA64
 * library.
 *
 * Whether a function is marked for static use, DLL export, or DLL import
 * is controlled by the VISIBILITY define.  With the MS VC 6.0 compiler
 * some acceptable values are:
 *
 *      #define VISIBILITY
 *      #define VISIBILITY      __declspec( dllexport)
 *      #define VISIBILITY      __declspec( dllimport) 
 *
 *
 * The calling convention is controlled by the CALLCONV define.  With the
 * MS VC 6.0 compiler some acceptable values are:
 *
 *      #define CALLCONV
 *      #define CALLCONV        __stdcall
 *
 *
 * A null VISIBILITY string would indicate standard static usage, while a
 * null CALLCONV string indicates C calling convention.
 *
 * We assume most users will want to use the library as a DLL from a
 * variety of languages, and so have defaulted to dllimport and the
 * stdcall convention below.
 *
 * Users needing to override these defaults should give suitable defines
 * from inside their source code or the compiler command line.
 *
 *
 *
 */


#define VOID                    void
#define INT                     int
#define UINT                    unsigned int
#define I16                     short
#define I32                     long
#define U16                     unsigned short
#define U32                     unsigned long


#ifdef SROS_LINUX
#define _STATIC 1
#undef _DLLEXPORT
#undef _DLLIMPORT
#endif


#ifdef _STATIC
#define VISIBILITY
#define CALLCONV
#endif

#ifdef _DLLEXPORT
#define VISIBILITY              __declspec( dllexport)
#define CALLCONV                __stdcall
#endif

#ifdef _DLLIMPORT
#define VISIBILITY              __declspec( dllimport)
#define CALLCONV                __stdcall
#endif


#ifndef VISIBILITY
#define VISIBILITY              __declspec( dllimport)
#endif

#ifndef CALLCONV
#define CALLCONV                __stdcall
#endif


#define VARTYPE( type)          VISIBILITY type
#define FUNCTYPE( type)         VISIBILITY type CALLCONV





/* RELEASE REV INFO:
 *
 * This define, variable, and function allow the user to determine if the
 * library rev they at either compile or runtime.
 *
 * The rev is an integer, where a number like 134 would be referred to
 * as rev "1.34".
 *
 *
 */

#define DSPA64_REV              231

extern VARTYPE( INT) DSPA64_Rev;

FUNCTYPE( INT) DSPA64_GetRev( void);






/* Keyboard helper functions */

#define KEY_CHECK 0
#define KEY_WAIT  1

FUNCTYPE( INT) GetKey( int wait );
FUNCTYPE( INT) GetKeyCheck( void );
FUNCTYPE( INT) GetKeyWait(  void );








/* DSPA64 SYSTEM DEFINES:
 *
 * The DSPA64 boards ship with a 10 Mhz crystal and allow
 * acquisition on up to sixty-four channels.
 *
 * The min and max number of ticks indicate how many clock ticks are
 * required by the A/D chip to perform a successful conversion.
 * 
 *
 */

#define XTAL            10.0E+06F
#define MAXCHANNELS     64
#define MIN_NTICKS      72
#define MAX_NTICKS      4097





/* DSPA64 BASE ADDRESS FUNCTIONS:
 *
 * These functions control the I/O space base address used when
 * communicating with the DSPA64.
 *
 */

FUNCTYPE( VOID) DSPA64_SetBaseIo( unsigned value);
FUNCTYPE( INT)  DSPA64_GetBaseIo( void);


/* These two just exist to provide backward compatibility for 3rd party apps. */

FUNCTYPE( VOID) DSPA64_SetBaseio( unsigned value);
FUNCTYPE( INT)  DSPA64_GetBaseio( void);





/* DSPA64 CONTROL/EXECUTION FUNCTIONS:
 *
 * These functions control the operation of the DSPA64.
 *
 * Use the helper functions below for setting DSPA64_Init parameters.
 *
 *
 */

FUNCTYPE( INT)  DSPA64_Init(

                      int   nchannel,
                      int   nsample,
                      int   ncount,
                      float filter_coeff,
                      float filter_scale,
                      int  *scan_list

                     );


FUNCTYPE( VOID) DSPA64_Start( void );
FUNCTYPE( VOID) DSPA64_Stop( void );
FUNCTYPE( VOID) DSPA64_ClearReady( void );
FUNCTYPE( INT)  DSPA64_IsReady( void );
FUNCTYPE( INT)  DSPA64_GetData( int status, short int *pcbuffer, int nbytes );


FUNCTYPE( VOID) DSPA64_BufferSize( int nchannel, int nsample, int *nbytes );
FUNCTYPE( VOID) DSPA64_SpsToNCount( int nchannel, float sps, int *ncount );
FUNCTYPE( VOID) DSPA64_NCountToSps( int nchannel, int ncount, float *sps );
FUNCTYPE( VOID) DSPA64_TailToCoeff( int ncount, float tail, float *coeff );
FUNCTYPE( VOID) DSPA64_CoeffToTail( int ncount, float coeff, float *tail );
FUNCTYPE( VOID) DSPA64_NormalizeScale( float coeff, float *scale );
FUNCTYPE( VOID) DSPA64_NTimingTicks( int newticks, int *nticks );
FUNCTYPE( VOID) DSPA64_MaximumRate( float *maxrate );

FUNCTYPE( INT)  DSPA64_Open( void );
FUNCTYPE( INT)  DSPA64_Close( void );
