/* FILE: simpa64.c              Copyright (c), Symmetric Research, 2003
 *
 * Acquire DSPA64 data with a simple text only display and disk save.
 *
 *
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#include <time.h>
#include <fcntl.h>        // for WriteTmpFile O_xxx defines
#include <sys/stat.h>    // for S_IREAD define (VC++ can use / instead of \)

#ifndef SROS_LINUX
#include <io.h>
#endif


#include "dspa64.h"
#include "simpa64.h"



char   msg[80];
struct INI_data INI = {  // <<< see simpa64.h for data layout ...


        // initializer values ...

        NULL,

        0,
        0,
        0,


        // ... since ChannelId[i] = 0 valid, set to -1 ...

        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,

        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,
        -1, -1, -1, -1, -1, -1, -1, -1,


        // ... let all other fields go to compiler default null ...

        };




/* STRING HELPER:
 *
 *
 *
 */

char *Chop( char *s ) {

        char *p;


        // Chop spaces ' ' off the end of a string and add '\n'.

        p = s;

        while ( *p )

                p++;


        if ( *--p == '\n' ) {

                while ( *--p == ' ' )

                        ; // back up


                *++p = '\n';
                *++p = '\0';

                }


        return( s );
}

void DisplayInfo( char *info ) {

        if (INI.InfoDisplay == ON)
                printf( info );
}




/* REV MESSAGE:
 *
 *
 *
 */

void Rev( void ) {

        // Print the rev info.

        DisplayInfo( "DSPA64 simpa64 : (c) SR 1999-2002 : Rev {07/05/02}\n" );

}






/* ERRORS:
 *
 * All errors except for .ini parsing errors go through these functions.
 *
 *
 */

char *UsageMsg[] = {

        "COMMAND LINE ERROR: %s                                      \n",
        "                                                            \n",
        "  simpa64 file.ini     ... where a .ini file is required    \n",

        NULL,

        };

void ErrorCommandLine( char *msg ) {


        // Print the command line usage message and quit.

        printf( Chop( UsageMsg[0] ), msg                     );
        printf( Chop( UsageMsg[1] )                          );
        printf( Chop( UsageMsg[2] )                          );

        exit( 1 );

}

#define ERROR_NOT_FATAL                 0
#define ERROR_FATAL                     1
#define WarningMsg( msg )               ErrorRuntime( msg, ERROR_NOT_FATAL )
#define ErrorFatal( msg )               ErrorRuntime( msg, ERROR_FATAL )
#define ErrorMaybeFatal( msg )          ErrorRuntime( msg, INI.ExitOnError )

void ErrorRuntime( char *msg, int fatal ) {

        FILE *ErrFp;

            char *datestr, dbuffer[16];
            char *timestr, tbuffer[16];
            time_t     current_time;
            struct tm *local_time;



        // Get the date and time.

        time( &current_time );
        local_time = localtime( &current_time );

        strftime( dbuffer, 16, "%m/%d/%y", local_time );
        strftime( tbuffer, 16, "%H:%M:%S", local_time );

        datestr = dbuffer;
        timestr = tbuffer;



        // Append the error to simpa64.err.

        if ( ErrFp = fopen( "SIMPA64.ERR", "a" ) ) {

                fprintf( ErrFp, "[%s] [%s] : %s\n", datestr, timestr, msg );

                   fclose( ErrFp );
                }



        // Display the error and quit.

        printf( "ERROR[simpa64]: %s\n", msg );



        // Quit if a fatal error.

        if ( fatal ) {

                DSPA64_Stop( );
                DSPA64_Close( );
                
                exit( 1 );
                }

}








/* INI DATA AND REPORTING:
 *
 *
 *
 */

void IniCompute( void ) {

        int NCount; float TrueSamplingRate;


        // Fill in the additional computed INI parameters.

        DSPA64_SpsToNCount( (int)INI.Channels, INI.SamplingRate, &NCount );
        DSPA64_NCountToSps( (int)INI.Channels, NCount, &TrueSamplingRate );


        INI.AcqBufSize         = INI.Channels * INI.ChBufPts * 2;
        INI.OutputFileSize     = INI.Blocks * INI.AcqBufSize;
        INI.TriggerTimeToSave  = (int)((float)(INI.TriggerFilesToSave * 
                                 INI.Blocks * INI.ChBufPts) /
                                 TrueSamplingRate);

        INI.NCount             = NCount;
        INI.Oversampling       = NCount+1;
        INI.TrueSamplingRate   = TrueSamplingRate;
        INI.BufferTime         = INI.ChBufPts / TrueSamplingRate;

        INI.FilterScale        = 1.0 - INI.FilterCoeff;
        INI.FilterDecay        = pow( INI.FilterCoeff, INI.Oversampling );


}

char *IniReportFmt[] = {

        "; DSPA64 INI DATA from [%s]:                            \n",
        "                                                        \n",
        "          Channel %02d = \"%s\"   Trigger = %s          \n",
        "                                                        \n",
        "            ChBufPts = %d                               \n",
        "        SamplingRate = %f                               \n",
        "         FilterCoeff = %g                               \n",
        "                                                        \n",
        "        SystemStatus = %s                               \n",
        "        Dsp32cBaseio = 0x%X                             \n",
        "       KeypressToRun = %s                               \n",
        "         ExitOnError = %s                               \n",
        "         DisplayData = %s                               \n",
        "         DisplayInfo = %s                               \n",
        "      OutputFileMode = %s                               \n",
        "      OutputSeqStart = 0x%X                             \n",
        "        OutputBlocks = %lu                              \n",
        "         TriggerType = %s                               \n",
        "  TriggerFilesToSave = %d                               \n",
        "    TriggerThreshold = %lu                              \n",
        "                                                        \n",
        "                                                        \n",
        "                                                        \n",
        "; Additional items implied by these parameters:         \n",
        ";                                                       \n",
        ";              Channels = %d                            \n",
        ";            AcqBufSize = %d bytes                      \n",
        ";        OutputFileSize = %lu bytes                     \n",
        ";     TriggerTimeToSave = %d seconds                    \n",
        ";                                                       \n",
        ";                NCount = %d                            \n",
        ";          Oversampling = %d times                      \n",
        ";      TrueSamplingRate = %f Hz                         \n",
        ";            BufferTime = %f sec                        \n",
        ";                                                       \n",
        ";           FilterScale = 1.0 - FilterCoeff         = %g\n",
        ";           FilterDecay = FilterCoeff^Oversampling  = %g\n",
        ";                                                       \n",

        NULL,

        };

void IniReport( void ) {

        FILE *RptFp;

            char **fmt; int i;

                char
                      *OutputFileMode( int value ),
                         *TriggerType( int value ),
                               *OnOff( int value );

                         


        // Generate the ASCII INI report file, simpa64.rpt.

        RptFp = fopen( "SIMPA64.RPT", "w" );

        if ( !RptFp )

                ErrorFatal( "unable to open simpa64.rpt report file" );



        fmt = &IniReportFmt[0];
        
        fprintf( RptFp, Chop( *fmt++ ), INI.Filename                   );
        fprintf( RptFp, Chop( *fmt++ )                                 );


        for ( i = 0 ; i < INI.Channels ; i++ )

         fprintf( RptFp, Chop( *fmt ),INI.ChannelId[i], INI.ChannelTitle[i],
                                               OnOff( INI.ChannelTrigger[i] ));

        fmt++;


        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.ChBufPts         );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.SamplingRate     );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.FilterCoeff      );
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ ),          OnOff( INI.SystemStatus   ) );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.Dsp32cBaseio     );
        fprintf( RptFp, Chop( *fmt++ ),          OnOff( INI.KeypressToRun  ) );
        fprintf( RptFp, Chop( *fmt++ ),          OnOff( INI.ExitOnError    ) );
        fprintf( RptFp, Chop( *fmt++ ),          OnOff( INI.DataDisplay    ) );
        fprintf( RptFp, Chop( *fmt++ ),          OnOff( INI.InfoDisplay    ) );
        fprintf( RptFp, Chop( *fmt++ ), OutputFileMode( INI.OutputFileMode ) );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.OutputSeqStart   );        fprintf( RptFp, Chop( *fmt++ ),                 INI.Blocks           );
        fprintf( RptFp, Chop( *fmt++ ),    TriggerType( INI.TriggerType )    );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.TriggerFilesToSave);
        fprintf( RptFp, Chop( *fmt++ ),                 INI.TriggerThreshold );
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ )                                       );

        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.Channels         );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.AcqBufSize       );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.OutputFileSize   );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.TriggerTimeToSave);
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.NCount           );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.Oversampling     );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.TrueSamplingRate );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.BufferTime       );
        fprintf( RptFp, Chop( *fmt++ )                                       );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.FilterScale      );
        fprintf( RptFp, Chop( *fmt++ ),                 INI.FilterDecay      );
        fprintf( RptFp, Chop( *fmt++ )                                       );


        if ( fclose( RptFp ) != 0 )

                ErrorFatal( "unable to close simpa64.rpt report file" );


}
char *OutputFileMode( int value ) {

        char *s;

        switch ( value ) {

                case NONE:         s = "NONE";        break;
                case SINGLE:       s = "SINGLE";      break;
                case SEQUENTIAL:   s = "SEQUENTIAL";  break;
                case TIME:         s = "TIME";        break;

                default:

                        s = "unknown OutputFileMode value"; break;
                }

        return( s );

}
char *TriggerType( int value ) {

        char *s;

        switch ( value ) {

                case TRIG_NONE:         s = "NONE";           break;
                case TRIG_THRESHOLD:    s = "THRESHOLD";      break;
                case TRIG_DIGIO:        s = "DIGITALSIGNAL";  break;

                default:

                        s = "unknown TriggerType value"; break;
                }

        return( s );

}
char *OnOff( int value ) {

        return( value ? "ON" : "OFF" );

}











/* HDR DATA AND REPORTING:
 *
 *
 *
 */

struct HDR_data HDR;  // <<< see simpa64.h for data layout ...


void HdrReport( void ) {

        FILE *HdrFp; int i;



        // Generate the binary HDR report file, simpa64.dhr.

        strcpy( HDR.Id, "SR DSPA64" );

        HDR.Channels       = INI.Channels;
        HDR.RecordPts      = INI.ChBufPts * INI.Blocks;
        HDR.WordSize       = 2;
        HDR.DataCoding     = DATACODING_SIGNED;

        HDR.SamplingRate   = INI.TrueSamplingRate;
        HDR.Gain           = 1.0;
        HDR.FilterCoeff    = INI.FilterCoeff;
        HDR.FilterScale    = INI.FilterScale;


        for ( i = 0 ; i < INI.Channels ; i++ ) {

                HDR.ChannelList[i].channel = INI.ChannelId[i];

                    strncpy(

                           HDR.ChannelList[i].title,
                           INI.ChannelTitle[i],
                           MAXTITLE-1

                          );
                }




        // Write the .hdr file.

        HdrFp = fopen( "SIMPA64.HDR", "wb" );

        if ( !HdrFp )

                ErrorFatal( "unable to open simpa64.hdr header file" );

        if ( fwrite( &HDR, sizeof( HDR ), 1, HdrFp ) != 1 )

                ErrorFatal( "unable to write simpa64.hdr header file" );


        fclose( HdrFp );

}






/* START MESSAGE:
 *
 *
 *
 */

char *StartMsgFmt[] = {

        "Initializing DSPA64 with: (see simpa64.rpt for more details)    \n",
        "                                                                \n",
        "     TrueSamplingRate = %f Hz                                   \n",
        "     (total) Channels = %d                                      \n",
        "          TriggerType = %s                                      \n",
        "         SystemStatus = %s                                      \n",
        "                                                                \n",

        NULL,

        };



void StartMsg( void ) {

        char **fmt;

        if (INI.InfoDisplay == OFF)
                return;

        // Display a few of the initialization parameters.

        fmt = StartMsgFmt;

            printf( Chop( *fmt++ )                                    );
            printf( Chop( *fmt++ )                                    );
            printf( Chop( *fmt++ ),              INI.TrueSamplingRate );
            printf( Chop( *fmt++ ),              INI.Channels         );
            printf( Chop( *fmt++ ), TriggerType( INI.TriggerType   )  );
            printf( Chop( *fmt++ ),       OnOff( INI.SystemStatus  )  );
            printf( Chop( *fmt++ )                                    );

}






/* KEYBOARD FUNCTIONS:
 *
 *
 *
 */

void KeyWait( void ) {

        int c;


        // Wait for a keypress, quitting if requested.

        c = GetKeyWait( );

        if ( c == 'q' || c == 'Q' || c == 0x3 ) { // exit on q, Q or Ctrl+C

                DSPA64_Stop( );
                DSPA64_Close( );

                exit( 0 );
                }


}
void KeyCheck( void ) {

        int c;

        // When running interactively, check for a keypress.

        if ( INI.KeypressToRun == ON ) {

                c = GetKeyCheck( );

                if ( c == 'q' || c == 'Q' )

                        { DSPA64_Stop( ); DSPA64_Close( ); exit( 0 ); }
                }
}







/* SCREEN DATA DISPLAY:
 *
 * Print out acquired data values to the screen.
 *
 * The DSPA64 acquires too much data to make printing all of it out in
 * text format resonable.  This function displays only a portion.
 *
 *
 *
 *
 */

#define DISPLAY_LINES           15
#define MAX_CHAN_DISPLAY        10

void DisplayValues( short *values ) {

        char *trailer;

                int chan_display, line, i; short *v;




        // Compute the number of channels to display.

        if ( INI.Channels > MAX_CHAN_DISPLAY )

                { chan_display = MAX_CHAN_DISPLAY; trailer = "  ...\n"; }

        else 
                { chan_display = INI.Channels; trailer = "\n"; }
        




        // Display the data.

        printf( "Acquired data (partial) ... \n\n" );

        for ( line = 0 ; line < DISPLAY_LINES ; line++ ) {


                v = &values[ INI.Channels * line];

                for ( i = 0 ; i < chan_display ; i++ )

                        printf( "  %04hX", *v++ );


                printf( trailer );


                }


        printf( "\n" );

}








/* DISK FUNCTIONS:
 *
 * These functions handle writing binary data output files.
 *
 *
 */

#ifdef SROS_LINUX
#define O_BINARY O_FSYNC
#endif

#define OFLAG           (O_BINARY|O_RDWR)
#define PMODE           (S_IREAD|S_IWRITE)

void WriteTmpFile( short *buffer, int nbytes, unsigned long block ) {

        int OutputFh, oflag;



        // Set the open flag to create or append depending on the block.

        if ( block == 0 )

                oflag = OFLAG | (O_CREAT|O_TRUNC);

        else
                oflag = OFLAG | O_APPEND;




        // Write the .tmp output file.

        OutputFh = open( "SIMPA64.TMP", oflag, PMODE );

        if ( OutputFh == -1 )

                ErrorMaybeFatal( "unable to open simpa64.tmp" );

        if ( write( OutputFh, buffer, nbytes ) != nbytes )

                ErrorMaybeFatal( "simpa64.tmp write failed" );

        if ( _commit( OutputFh ) != 0 )

                ErrorMaybeFatal( "unable to commit simpa64.tmp to disk" );

        if ( close( OutputFh ) != 0 )

                ErrorMaybeFatal( "unable to close simpa64.tmp" );


}
void RenameTmpFile( int OutputFileMode, unsigned long nfile ) {

        char FinalName[16];



        // Compute the final .out file name.

        if ( OutputFileMode == SINGLE )

                sprintf( FinalName, "SIMPA64.OUT" );

        else if ( OutputFileMode == SEQUENTIAL )

                sprintf( FinalName, "%08lX.OUT", nfile );

        else if ( OutputFileMode == TIME )

                sprintf( FinalName, "%08lX.OUT", time( NULL ) );

        else // OutputFileMode == NONE

                return;



#ifdef SROS_LINUX

        // Verify that new name is available.

        if ( access( FinalName, F_OK ) == 0 ) {

                sprintf( msg, "Rename failed.  File %s already exists.",
                                                                   FinalName );
                ErrorMaybeFatal( msg );
                }
#endif



        // Rename the .tmp file.

        if ( rename( "SIMPA64.TMP", FinalName ) == 0 ) {

                sprintf( msg, "simpa64.tmp renamed to %s ... \n", FinalName );
                DisplayInfo( msg );
                }

        else
                ErrorMaybeFatal( "simpa64.tmp rename failed" );


}
void SaveToDisk( short *PC_BUFFER, unsigned long nbuffer,
                 unsigned long *ntriggerfile ) {

        static unsigned long block = 0;
        static unsigned long nfile = 0;


        // If no disk save, just return.

        if ( INI.OutputFileMode == NONE )

                { printf( "Buffer[%08lX] processed\n", nbuffer ); return; }


        
        // If no trigger, just returnl
        
        if ( ! TriggerCondition( PC_BUFFER, ntriggerfile ) ) {
                
                sprintf( msg, "Buffer[%08lX] discarded\n", nbuffer );
                DisplayInfo( msg );
                return;
                }
                
        

        // Otherwise, save to the .tmp file ...

        WriteTmpFile( PC_BUFFER, (int)INI.AcqBufSize, block );

        sprintf( msg, "Buffer[%08lX] block %02lu -> simpa64.tmp\n", 
                                                            nbuffer, block );
        DisplayInfo( msg );



        // Increment the block count and check for reset.

        if ( ++block >= INI.Blocks ) {

                RenameTmpFile( INI.OutputFileMode, INI.OutputSeqStart+nfile );

                nfile++;
                block = 0;
                if (*ntriggerfile > 0)
                        *ntriggerfile = *ntriggerfile - 1;
                }

}

void TriggerThresholdSetRange( short *PC_BUFFER, long ch,
                               short *ThMin, short *ThMax ) {
                              
        double TMPoffset;
        short  DCoffset, j, k;

        // Find channel offset

        k         = ch;
        TMPoffset = 0;
        
        for ( j = 0 ; j < INI.ChBufPts ; j++ ) {
                TMPoffset += PC_BUFFER[k];
                k         += INI.Channels;
                }
        DCoffset = (short)(TMPoffset / (double)INI.ChBufPts);


        // Compute threshold range from offset

        *ThMin = DCoffset - (short)INI.TriggerThreshold;
        if (*ThMin > DCoffset)
                *ThMin = (short)0x8000; // min for 16 bit signed data = -32768

        *ThMax = DCoffset + (short)INI.TriggerThreshold;
        if (*ThMax < DCoffset)
                *ThMax = (short)0x7FFF; // max for 16 bit signed data = 32767
}

int TriggerCondition( short *PC_BUFFER, unsigned long *ntriggerfile ) {

        short i, j, k, ThresholdMax, ThresholdMin;

        // Continuous recording

        if ( INI.TriggerType == TRIG_NONE )
                return( 1 );


        // Amplitude threshold

        else if (INI.TriggerType == TRIG_THRESHOLD) {

                // Compute the amplitude threshold trigger condition

                for ( i = 0 ; i < INI.Channels ; i++ ) {

                    if (INI.ChannelTrigger[i] == ON) {

                        TriggerThresholdSetRange( PC_BUFFER, i,
                                                  &ThresholdMin,
                                                  &ThresholdMax );

                        k = i;
                        for ( j = 0 ; j < INI.ChBufPts ; j++ ) {

                            if ( (PC_BUFFER[k] < ThresholdMin) ||
                                 (PC_BUFFER[k] > ThresholdMax) ) {

                                    // If already saving, don't count current
                                    // partial file

                                    if (*ntriggerfile > 0)
                                            *ntriggerfile = INI.TriggerFilesToSave + 1;
                                    else
                                            *ntriggerfile = INI.TriggerFilesToSave;

                                    sprintf( msg, "Amplitude exceeds threshold\n" );
                                    DisplayInfo( msg );
                                    return( 1 );
                                    } // end if > Threshold

                                k += INI.Channels;

                            } // end for j
                        } // end if ON
                    } // end for i
                } // end TRIG_THRESHOLD



        // Digital I/O line signal
        
        else if (INI.TriggerType == TRIG_DIGIO) {

                WarningMsg( "Digital I/O trigger type not supported" );
                return( 0 );
                
                } // end TRIG_DIGIO


        // Unknown

        else {
                WarningMsg( "Unknown trigger type" );
                return( 0 );
                }


        // A positive trigger file count may come from either
        //   1) An amplitude exceeding the threshold in the current buffer
        //   2) More files still remaining to be written after a
        //      previous high amplitude was found

        if (*ntriggerfile > 0)
                return( 1 );
        else
                return( 0 );

}




/* SLEEP FUNCTION:
 *
 *
 *
 *
 */

int SrSleep( int ms ) {

        /* Sleep for specifed number of milliseconds.
         * Returns 0 for success, -1 for error.
         *
         * Linux nanosleep takes structure with seconds and nanoseconds.
         * No sleep function is available for MSDOS, so return immediately.
         */

#ifdef SROS_LINUX
        
        struct timespec twait, tleft;
        tleft.tv_sec  = tleft.tv_nsec = 0;
        twait.tv_sec  = (long)(ms / 1000);
        twait.tv_nsec = (ms - (twait.tv_sec*1000)) * 1000000;
        return( nanosleep( &twait, &tleft ) );

#elif defined( SROS_MSDOS )
        
        return( 0 );

#else        
        _sleep( ms );
        return( 0 );
#endif
        
}








/* MAIN:
 *
 *
 *
 *
 */

int main( int argc, char *argv[] ) {

        int i, status, sleepms;
        short *PC_BUFFER;
        unsigned long nbuffer, ntriggerfile; 



#ifdef SROS_LINUX

        // Allow getchar to return even if no character is waiting.

        if ( SetNonBlock( 1 ) == -1 ) 
                ErrorMaybeFatal( "Can't set non-blocking read ..." );

#endif



        // Check the number of command line arguments.

        if ( argc != 2 )

                ErrorCommandLine( "wrong number of command line arguments" );




        // Setup default INI parameters.

        INI.Filename           = argv[1];

        INI.ChBufPts           = 1500;
        INI.SamplingRate       = 100.0;
        INI.FilterCoeff        = 0.95;
        INI.Dsp32cBaseio       = 0x280;

        INI.SystemStatus       = ON;
        INI.KeypressToRun      = ON;
        INI.ExitOnError        = ON;
        INI.DataDisplay        = ON;
        INI.InfoDisplay        = ON;
        INI.OutputFileMode     = SINGLE;
        INI.Blocks             = 2;
        INI.TriggerType        = NONE;
        INI.TriggerFilesToSave = 2;
        INI.TriggerDigitalLine = 0;
        INI.TriggerThreshold   = 0L;

        INI.Channels = 4;   // ... set up a default channel list ...


        for ( i = 0 ; i < INI.Channels ; i++ ) {
                INI.ChannelId[i] = i;
                INI.ChannelTrigger[i] = OFF;
                }
                
            INI.ChannelTitle[0] = "CH00";
            INI.ChannelTitle[1] = "CH01";
            INI.ChannelTitle[2] = "CH02";
            INI.ChannelTitle[3] = "CH03";




        // Parse the user's .ini file.

        ProcessIni();



        // Display the rev.

        Rev();



        // Fill in the additional computed INI parameters ...

        IniCompute();



        // Generate the .rpt and .hdr files.

        DisplayInfo( "Generating simpa64.rpt ... \n" );
            
                IniReport();

        DisplayInfo( "Generating simpa64.hdr ... \n" );

                HdrReport();

        DisplayInfo( "\n" );






        // Initialize the DSPHLF/DSPA64.

        StartMsg();

#define INIT_SUCCESS   "DSPA64_Init successful\n"
#define INIT_FAILURE   "DSPA64_Init failed, check system settings and power"
#define INIT_SKIPPED   "DSPA64_Init skipped, SystemStatus = OFF\n"

        if ( INI.SystemStatus == ON ) {

                DSPA64_SetBaseIo( INI.Dsp32cBaseio );

                status = DSPA64_Init(
                                        (int)INI.Channels,
                                        (int)INI.ChBufPts,
                                        (int)INI.NCount,
                                        INI.FilterCoeff,
                                        INI.FilterScale,
                                        INI.ChannelId

                                        );

                if ( status == 1 ) {

                        sprintf( msg, INIT_SUCCESS );
                        DisplayInfo( msg );
                        }

                else
                        ErrorFatal( INIT_FAILURE );

                }

        else
                DisplayInfo( INIT_SKIPPED );


        DisplayInfo( "\n" );






        /* Allocate PC acquisition buffer memory:
         *
         * This PC memory is used to store data on its way out from the
         * DSPHLF/DSPA64 to the disk and screen.
         *
         *
         */

        PC_BUFFER = (short *)malloc( (int)INI.AcqBufSize );

        if ( !PC_BUFFER )

                ErrorFatal( "unable to malloc PC buffer memory" );





        // Initialize various parameters.

        nbuffer      = 0;
        ntriggerfile = 0;


        // Initialize amount of sleep time as 
        // (buffertime sec / 10) * 1000 ms/sec

        sleepms = INI.BufferTime*100; 



        /* Start executing:
         *
         * The DSP32C executes continuously, saving data in its on board
         * memory.
         *
         * Note that the DSP's memory buffer is twice the size of the
         * PC buffer so acquisition continues uninterrupted while the PC
         * saves to disk.
         *
         *
         *
         */

        if ( INI.KeypressToRun == ON ) {

                printf( "Hit ENTER to start, Ctrl+C to quit ... \n" );

                KeyWait();

                printf( "\n" );

                }


        if ( INI.SystemStatus == ON )

                DSPA64_Start();







        /* Poll to see if a DSPHLF buffer is ready.
         *
         * Everytime data is ready, IsReady will return a non zero value.
         *
         *
         *
         */

#define NOT_READY       0

        sprintf(msg,"Waiting %4.2f sec for first buffer ...\n",INI.BufferTime);
        DisplayInfo( msg );

polling_loop:


        // Is data ready?

        if ( INI.SystemStatus == ON ) {

                status = DSPA64_IsReady();

                if ( status == NOT_READY )

                        { KeyCheck(); SrSleep( sleepms ); goto polling_loop; }
                }




        // Data is ready, copy the DSPHLF buffer to PC memory.

        if ( INI.SystemStatus == ON )

                DSPA64_GetData( status, PC_BUFFER, (int)INI.AcqBufSize );





        // Save the buffer to the hard disk if trigger condition is met.

        SaveToDisk( PC_BUFFER, nbuffer, &ntriggerfile  );




        // Display the acquired data.

        if ( INI.DataDisplay == ON )

                DisplayValues( PC_BUFFER );



        // Increment the number of buffers processed.

        nbuffer++;



        // Has overflow occurred ?

        if ( (INI.SystemStatus == ON) && DSPA64_IsReady() ) {

                ErrorMaybeFatal( "data overflow during PC processing ..." );

                // If error was not fatal, reset and continue
                
                DSPA64_Stop();
                DSPA64_Start();
	        }



        // Check for a 'q' to quit, and then continue polling.

        KeyCheck();

    goto polling_loop;


    return( 0 );
}
















/* INI FILE PARSING:
 *
 * These functions parse the .ini input file.
 *
 * They are secondary to the main flow of the program.  Study main() above
 * for the core acquisition steps.
 *
 * These functions are more complicated than simple fscanf to allow for
 * free format syntax and better error reporting.
 *
 *
 *
 */

FILE *IniFp;

int
        fline = 1;



#define MAXTOKEN        256             // token info

char TOKEN[ MAXTOKEN], *token;

#define SYMBOL          1               // token types
#define EQUALS          2
#define HEX             3
#define INTEGER         4
#define FLOAT           5
#define STRING          6
#define UNKNOWN         7

int ttype;




void ErrorIni( char *msg ) {


        // Print an INI parsing error message and quit.

        printf( "ERROR[%s] : line %d : %s", INI.Filename, fline, msg );

        exit( 1 );


}
void AddToToken( int c ) {


        // Add c to TOKEN, checking for token buffer overflow.

        if ( (token - TOKEN) < MAXTOKEN )

                *token++ = c;

        else
                ErrorIni( "internal token length exceeded" );


}
int SkipSpace( int c ) {


        // Skip past white space, including ; comments.
skip:

        if ( isspace( c ) ) {

                if ( c == '\n' )

                        fline++;


                c = fgetc( IniFp );

                goto skip;

                }


        else if ( c == ';' ) { // ; comment ...

                while ( c != '\n' && c != EOF )

                        c = fgetc( IniFp );

                goto skip;

                }


        return( c );


}
int GetToken( void ) {

        static int c = ' ';



        // Skip over any white space.

        c = SkipSpace( c );



        // Gather the next token.

        token = TOKEN;


        if ( isalpha( c ) || c == '_' ) {

                ttype = SYMBOL;

                while ( isalnum( c ) || c == '_' || c == '/' )

                        { AddToToken( c ); c = fgetc( IniFp ); }
                }


        else if ( c == '=' ) {

                ttype = EQUALS;

                  AddToToken( c ); c = fgetc( IniFp );

                }


        else if ( isdigit( c ) ) {

                // ttype could be HEX, INTEGER, or FLOAT ...

                AddToToken( c );

                  c = fgetc( IniFp );



                if ( TOKEN[0] == '0' && c == 'x' ) {

                        ttype = HEX;
                        token = TOKEN;

                        c = fgetc( IniFp );

                        if ( isxdigit( c ) ) {

                                while ( isxdigit( c ) )

                                        { AddToToken( c ); c = fgetc( IniFp ); }
                                }

                        else
                                ttype = UNKNOWN;

                        }



                else {

                        ttype = INTEGER;

                        while ( isdigit( c ) )

                                { AddToToken( c ); c = fgetc( IniFp ); }



                        if ( c == '.' ) {

                                ttype = FLOAT;

                                AddToToken( c );

                                  c = fgetc( IniFp );

                                while ( isdigit( c ) )

                                        { AddToToken( c ); c = fgetc( IniFp ); }
                                }
                        }
                }



        else if ( c == '"' ) {

                ttype = STRING;

                c = fgetc( IniFp );

                  while ( c != '"' && c != '\n' && c != EOF )

                        { AddToToken( c ); c = fgetc( IniFp ); }



                if ( c == '"' )

                        c = fgetc( IniFp );

                else
                        ErrorIni( "missing quote on string" );

                }



        else if ( c == EOF )

                ttype = EOF;


        else {

                ttype = UNKNOWN;

                  AddToToken( c );


                c = fgetc( IniFp );

                }




        // Null terminate the TOKEN and return its type.

        AddToToken( '\0' );

        return( ttype );



}
int Match( char *s ) {

        char *t;


        // Check for a match between s and the current token.

        t = TOKEN;

        while ( *s && *t && toupper( *s ) == toupper( *t ) )

                { s++; t++; }


        return ( *s == '\0' && *t == '\0' ? 1 : 0 );



}
int KeywordEquals( char *keyword ) {

        char msg[80];

#define KEYWORD_EQUALS_ERROR   "expected = sign after %s keyword"



        if ( Match( keyword ) ) {


                if ( GetToken() == EQUALS )

                        return( 1 );

                else {
                        sprintf( msg, KEYWORD_EQUALS_ERROR, keyword );

                          ErrorIni( msg );

                        }
                }



        return( 0 );


}
void GetOnOff( int *parameter, char *error ) {


        // Get an {ON,OFF} parameter value.

        GetToken();

        if ( Match( "ON" ) )

                *parameter = ON;

        else if ( Match( "OFF" ) )

                *parameter = OFF;

        else
                ErrorIni( error );


}
void GetInt( int *parameter, char *error ) {


        // Get a decimal int parameter value.

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error );


        sscanf( TOKEN, "%d", parameter );


}
void GetUint( unsigned int *parameter, char *error ) {


        // Get a decimal int parameter value.

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error );


        sscanf( TOKEN, "%u", parameter );


}
void GetUlong( unsigned long *parameter, char *error ) {

        // Get an unsigned long parameter value.

        GetToken();

        if ( ttype != INTEGER )

                ErrorIni( error );


        sscanf( TOKEN, "%lu", parameter );


}
void GetHex( int *parameter, char *error ) {

        // Get a hex parameter value.

        GetToken();

        if ( ttype != HEX )

                ErrorIni( error );


        sscanf( TOKEN, "%x", parameter );



}
void GetFloat( double *parameter, char *error ) {


        // Get a floating point parameter value.

        GetToken();

        if ( (ttype != FLOAT) && (ttype != INTEGER) )

                ErrorIni( error );


        sscanf( TOKEN, "%lf", parameter );


}
void AddChannel( int channel, char *title ) {

        static int firstcall = 1; int i;



        // If this is the first call, delete any default channel list.

        if ( firstcall )

                { INI.Channels = 0; firstcall = 0; }



        // Make sure the channel is in the range.

        if ( channel < 0 || MAXCHANNELS <= channel )

                ErrorIni( "channel number out of range" );



        // Search INI channel list to see if channel already listed.

        for ( i = 0 ; i < INI.Channels ; i++ ) {

                if ( INI.ChannelId[i] == channel  )

                        ErrorIni( "channel cannot be defined twice" );
                }



        // Add the channel to the list.

        if ( INI.Channels < MAXCHANNELS ) {

                INI.ChannelId[i] = channel;

                INI.ChannelTitle[i] = strdup( title );

                if ( !INI.ChannelTitle[i] )

                        ErrorIni( "out of memory, strdup failed!" );

                INI.ChannelTrigger[i] = OFF;

                INI.Channels++;

                }

        else
                ErrorIni( "tried to define too many channels" );



}
void ParseIni( void ) {

        int channel, gain, normalize;
        double dtmp, dtol;
        unsigned long MaxThreshold;
        static int LastChannel;

        // Parse the .ini file.

parse:
        ttype = GetToken();

	
        // Hardware keywords

        if ( KeywordEquals( "Dsp32cBaseio" ) ) {

                GetHex( &INI.Dsp32cBaseio, "illegal Dsp32cBaseio hex value" );

                if ( INI.Dsp32cBaseio < 0x0 )

                        ErrorIni( "Dsp32cBaseio must be > 0x0" );
                }


        else if ( KeywordEquals( "SystemStatus" ) )

                GetOnOff( &INI.SystemStatus, "SystemStatus must = {ON,OFF}" );


        
        // System keywords
        
        else if ( KeywordEquals( "KeypressToRun" ) )

                GetOnOff( &INI.KeypressToRun, "KeypressToRun must = {ON,OFF}" );


        else if ( KeywordEquals( "ExitOnError" ) )

                GetOnOff( &INI.ExitOnError, "ExitOnError must = {ON,OFF}" );



        // Acquisition keywords

        else if ( KeywordEquals( "SamplingRate" ) )

                GetFloat( &INI.SamplingRate, "SamplingRate must = float" );


        else if ( KeywordEquals( "Gain" ) ) {

                GetInt( &gain, "illegal Gain integer value" );
                if ( gain != 1 )
                        ErrorIni( "Gain must = 1" );
                }

        
        else if ( KeywordEquals( "Normalize" ) ) {

                GetOnOff( &normalize, "Normalize must = {ON,OFF}" );
                if (normalize != ON)
                        ErrorIni( "Normalize must = {ON}" );
                }
        

        else if ( KeywordEquals( "FilterCoeff" ) ) {

                GetFloat( &INI.FilterCoeff, "FilterCoeff must = float" );

                if ( INI.FilterCoeff < 0.00 || 1.0 < INI.FilterCoeff )

                        ErrorIni( "FilterCoeff must be in the range [0,1]" );
                
                INI.FilterScale = 1.0 - INI.FilterCoeff;
                }

        
        else if ( KeywordEquals( "FilterScale" ) ) {

                GetFloat( &INI.FilterScale, "FilterScale must = float" );

                if ( INI.FilterScale < 0.00 || 1.0 < INI.FilterScale )

                        ErrorIni( "FilterScale must be in the range [0,1]" );

                dtmp = 1.0 - INI.FilterCoeff;
                dtol = 0.0001;
                
                if ( INI.FilterScale < (dtmp-dtol) || (dtmp+dtol) < INI.FilterScale )

                        ErrorIni( "FilterScale must equal 1.0 - FilterCoeff" );
                }


        
        // Channel keywords

        else if ( Match( "Channel" ) ) {

                GetInt( &channel, "illegal CHANNEL number" );

                if ( GetToken() != EQUALS )

                        ErrorIni( "missing = sign on CHANNEL" );

                if ( GetToken() != STRING )

                        ErrorIni( "illegal CHANNEL title" );


                AddChannel( channel, TOKEN );
                LastChannel = channel;

                }

        



        // Trigger Keywords

        else if ( KeywordEquals( "Trigger") ) {

                if ( LastChannel < 0 || MAXCHANNELS <= LastChannel )
                        ErrorIni( "missing Channel keyword before trigger" );

                GetOnOff( &INI.ChannelTrigger[LastChannel],
                          "Trigger must = {ON,OFF}" );
                
                LastChannel = -1;
                }



        else if ( KeywordEquals( "TriggerType" ) ) {

                GetToken();

                if ( Match( "NONE" ) )

                        INI.TriggerType = TRIG_NONE;

                else if ( Match( "THRESHOLD" ) )

                        INI.TriggerType = TRIG_THRESHOLD;

                else if ( Match( "DIGITALSIGNAL" ) )

                        INI.TriggerType = TRIG_DIGIO;

                else
                    ErrorIni( "TriggerType must = {NONE,THRESHOLD,DIGITALSIGNAL}" );

                }


        else if ( KeywordEquals( "TriggerFilesToSave" ) ) {

                GetInt( &INI.TriggerFilesToSave,
                          "illegal TriggerFilesToSave integer value" );

                if ( INI.TriggerFilesToSave <= 0 )

                        ErrorIni( "TriggerFilesToSave must be > 0" );
                }


        else if ( KeywordEquals( "TriggerThreshold" ) ) {

                MaxThreshold = 32768L;
                GetUlong( &INI.TriggerThreshold,
                          "illegal TriggerThreshold integer value" );

                if ( (INI.TriggerThreshold <= 0) ||
                     (INI.TriggerThreshold > MaxThreshold)  ) {

                        sprintf( msg,
                                 "TriggerThreshold must be > 0 and less than max amplitude %d (0x%X)",
                                 MaxThreshold, MaxThreshold );
                        ErrorIni( msg );
                        }
                }


        else if ( KeywordEquals( "TriggerDigitalLine" ) ) {

                ErrorIni( "TriggerDigitalLine not supported for DSPA64" );

                }





        // Output file keywords
        
        else if ( KeywordEquals( "ChBufPts" ) ) {

                GetUlong( &INI.ChBufPts, "illegal ChBufPts integer value" );

                if ( INI.ChBufPts <= 0 )

                        ErrorIni( "ChBufPts must be > 0" );
                }


        else if ( KeywordEquals( "OutputFileMode" ) ) {

                GetToken();

                if ( Match( "NONE" ) )

                        INI.OutputFileMode = NONE;

                else if ( Match( "SINGLE" ) )

                        INI.OutputFileMode = SINGLE;

                else if ( Match( "SEQUENTIAL" ) )

                        INI.OutputFileMode = SEQUENTIAL;

                else if ( Match( "TIME" ) )

                        INI.OutputFileMode = TIME;
                else

                    ErrorIni(

                       "OutputFileMode must = {NONE,SINGLE,SEQUENTIAL,TIME}"

                              );
                }


        else if ( KeywordEquals( "OutputSeqStart" ) )

                GetHex( &INI.OutputSeqStart,
                        "illegal OutputSeqStart hex value" );
                        


        else if ( KeywordEquals( "OutputBlocks" ) ||
                  KeywordEquals( "Blocks" ) )

                GetUlong( &INI.Blocks, "illegal Blocks integer value" );


        
        // Display keywords

        else if ( KeywordEquals( "DisplayData" ) ||
                  KeywordEquals( "DataDisplay" ) )

                GetOnOff( &INI.DataDisplay, "DataDisplay must = {ON,OFF}" );


        else if ( KeywordEquals( "DisplayInfo" ) ||
                  KeywordEquals( "InfoDisplay" ) )

                GetOnOff( &INI.InfoDisplay, "InfoDisplay must = {ON,OFF}" );



        // Unused options
        // These options are not used in simp at this time, but they
        // are used by the scope acquisition program and we want
        // simp to work with the scope .ini files.

        else if ( KeywordEquals( "Title" )                ||
                  KeywordEquals( "ForbidMultiple" )       ||
                  KeywordEquals( "Interactive" )          ||
                  KeywordEquals( "JP60Setting" )          ||
                  KeywordEquals( "Overflow" )             ||
                  KeywordEquals( "OutputDirectory" )      ||
                  KeywordEquals( "DisplayColor" )         ||
                  KeywordEquals( "DisplayNbits" )         ||
                  KeywordEquals( "DisplayAutoDC" )        ||
                  KeywordEquals( "DisplayClipping" )      ||
                  KeywordEquals( "DisplayFirstSample" )   ||
                  KeywordEquals( "DisplayFirstTrace" )    ||
                  KeywordEquals( "DisplayNtraces" ) ) {
                
                GetToken();  // Ignore them
                
                }

        
        else if ( ttype == EOF )

                return;


        else // token unrecognized ...

                ErrorIni( "unknown simpa64.ini syntax" );




        // Continue parsing.

        goto parse;


}
int ProcessIni( void ) {


        // Open and parse the .ini file.

        IniFp = fopen( INI.Filename, "r" );

        if ( !IniFp )

                ErrorCommandLine( "unable to open .ini file" );



        ParseIni();

        fclose( IniFp );



        // Run any final .ini parameter checks.

        if ( !INI.Channels )

                ErrorIni( "at least one CHANNEL keyword is required" );

        
        return( 1 );
}
